/*
 * !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 *
 * This file is generated automatically from './src/pmc/complex.pmc'
 * by tools/build/pmc2c.pl.
 *
 * Any changes made here will be lost!
 *
 */

#line 1 "./src/pmc/complex.pmc"
/*
Copyright (C) 2004-2007, The Perl Foundation.
$Id: complex.pmc 18754 2007-06-02 03:29:34Z petdance $

=head1 NAME

src/pmc/complex.pmc - Complex Numbers PMC Class

=head1 DESCRIPTION

C<Complex> provides a representation of complex numbers. It handles
string parsing/generating and basic mathematical operations.

=head2 Functions

=over 4

=cut

Equations used are sometimes listed.  At times, multiple equations are given,
but those starting with => are the ones used

*/

#include "parrot/parrot.h"


#define RE(obj) (((FLOATVAL*)PMC_struct_val(obj))[0])
#define IM(obj) (((FLOATVAL*)PMC_struct_val(obj))[1])


/*

=item C<static void
complex_parse_string(Interp* interp,
        FLOATVAL *re, FLOATVAL *im, STRING *value)>

Parses the string in C<value> to produce a complex number, represented
by the real (C<*re>) and imaginary (C<*im>) parts. Raises an exception
if it cannot understand the string.  The string should be of the form
C<a+bi> with optional spaces around C<+> and before C<i>. You can also
use C<j> instead of C<i>.

=cut

*/

static void
complex_parse_string(Interp* interp,
        FLOATVAL *re, FLOATVAL *im, STRING *value) {
    char *str;
    INTVAL first_num_length, second_num_length;
    INTVAL first_num_minus, second_num_minus, i;
    char *first_num_offset, *second_num_offset = NULL, *t;
    STRING *S;

    t = str = string_to_cstring(interp, value);
    i = 0;
    first_num_offset = str;
    first_num_minus = second_num_minus = 0;

    /* walk the string and identify the real and imaginary parts */

    if (*t == '-') {
        /* first number is negative */
        t++;
        first_num_minus = 1;
        if (*t == ' ') t++; /* allow for an optional space */
        first_num_offset = t;
    }
    while (*t >= '0' && *t <= '9') t++; /* skip digits */
    if (*t == '.') {
        /* this number has a decimal point */
        t++;
        while (*t >= '0' && *t <= '9') t++; /* skip digits */
    }
    /* save the length of the real part */
    first_num_length = t - first_num_offset;

    if (*t == 0) {
        /* end of string; we only have a real part */
        second_num_length = 0;
    }
    else if ((*t == 'i' || *t == 'j') && *(t+1) == 0) {
        /* there is only an imaginary part, so the first number was
            actually the imaginary part */
        second_num_length = first_num_length;
        first_num_length = 0;
        second_num_offset = first_num_offset;
        second_num_minus = first_num_minus;
        first_num_minus = 0;
        /* this is useful if there is no number for
            the imaginary part, like in "-i" */
        i = 1;
    }
    else {
        if (*t == ' ') t++; /* skip an optional space */
        /* expect "+" or "-" and the imaginary part */
        if (*t == '+' || *t == '-') {
            second_num_minus = (*t == '-'); /* save the sign */
            t++;
            if (*t == ' ') t++; /* skip another optional space */

            /* save the beginning of the imaginary part */
            second_num_offset = t;
            while (*t >= '0' && *t <= '9') t++; /* skip digits */
            if (*t == '.') {
                /* this number has a decimal point */
                t++;
                while (*t >= '0' && *t <= '9') t++; /* skip digits */
            }
            /* save the length of the imaginary part */
            second_num_length = t - second_num_offset;

            if (*t == ' ') t++; /* allow for one more optional space */

            /* verify that the string ends properly */
            if ((*t != 'i' && *t != 'j') || (*(t+1) != 0)) {
                /* imaginary part does not end in 'i' or 'j' */
                real_exception(interp, NULL, E_ValueError,
                    "Complex: malformed string");
                first_num_length = second_num_length = 0;
            }
            /* this is useful if there is no number for the
                imaginary part, like in "2+i" */
            i = 1;

            /* all is OK, save the number */
        }
        else {
            /* "+" or "-" not found: error */
            real_exception(interp, NULL, E_ValueError,
                "Complex: malformed string");
            first_num_length = second_num_length = 0;
        }
    }

    /* now we have the offsets and the lengths;
        we turn them into float values */

    if (first_num_length) {
        /* there is a real part, interpret it */
        S = string_from_cstring(interp,
                first_num_offset, first_num_length);
        *re = string_to_num(interp, S);
    }
    else {
        /* consider the real part 0.0 */
        *re = 0.0;
    }

    if (second_num_length) {
        /* there is an imaginary part, interpret it */
        S = string_from_cstring(interp,
                second_num_offset, second_num_length);
        *im = string_to_num(interp, S);
    }
    else {
        /* consider the imaginary part 0.0 */
        if (i) /* the string was something like "1+i" */
            *im = 1.0;
        else
            *im = 0.0;
    }
    if (first_num_minus) *re = - *re;
    if (second_num_minus) *im = - *im;

    string_cstring_free(str);
}

/*

=item C<static FLOATVAL*
complex_locate_keyed_num(Interp* interp, PMC* self, STRING *key)>

Interpret the string C<key>; valid keys are C<real> and C<imag>,
representing the real and imaginary parts of the complex number.

=cut

*/

static FLOATVAL*
complex_locate_keyed_num(Interp* interp, PMC* self, STRING *key) {
    /* do imag first since real can be read much faster anyway */
    STRING *imag = string_from_cstring(interp, "imag", 4);
    STRING *real;

    if (0 == string_equal(interp, key, imag))
        return &IM(self);
    real = string_from_cstring(interp, "real", 4);
    if (0 == string_equal(interp, key, real))
        return &RE(self);
    real_exception(interp, NULL, E_KeyError,
        "Complex: key is neither 'real' or 'imag'");
    return NULL;
}

static void
int_check_divide_zero(Interp *interp, INTVAL value)
{
    if (value == 0) {
        real_exception(interp, NULL, E_ZeroDivisionError,
                       "Divide by zero");
    }
}

static void
float_check_divide_zero(Interp *interp, FLOATVAL value)
{
    if (value == 0.0) {
        real_exception(interp, NULL, E_ZeroDivisionError,
                       "Divide by zero");
    }
}

static void
complex_check_divide_zero(Interp *interp, PMC *value)
{
    /* Throw an exception if we are dividing by zero. */
    if (RE(value) == 0.0 && IM(value) == 0.0) {
        real_exception(interp, NULL, E_ZeroDivisionError,
                       "Divide by zero");
    }
}


#line 240 "src/pmc/complex.c"
#include "parrot/parrot.h"
#include "parrot/extend.h"
#include "parrot/dynext.h"
#include "pmc_complex.h"
#include "pmc_default.h"
#include "complex.str"

PMC*
Parrot_Complex_add(Interp *interp, PMC* pmc, PMC* value, PMC* dest)
#line 646 "./src/pmc/complex.pmc"
{
        if (dest)
            VTABLE_morph(interp, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interp, pmc->vtable->base_type);
        }
        RE(dest) = RE(pmc) + VTABLE_get_number(interp, value);
        IM(dest) = IM(pmc);
        return dest;
}
#line 261 "src/pmc/complex.c"
PMC*
Parrot_Complex_add_Complex(Interp *interp, PMC* pmc, PMC* value, PMC* dest)
{
        if (dest)
            VTABLE_morph(interp, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interp, pmc->vtable->base_type);
        }
        RE(dest) = RE(pmc) + RE(value);
        IM(dest) = IM(pmc) + IM(value);
        return dest;
}
PMC*
Parrot_Complex_add_int(Interp *interp, PMC* pmc, INTVAL value, PMC* dest)
#line 669 "./src/pmc/complex.pmc"
{
    if (dest)
        VTABLE_morph(interp, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interp, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) + value;
    IM(dest) = IM(pmc);
    return dest;

}
#line 288 "src/pmc/complex.c"
PMC*
Parrot_Complex_add_float(Interp *interp, PMC* pmc, FLOATVAL value, PMC* dest)
#line 680 "./src/pmc/complex.pmc"
{
    if (dest)
        VTABLE_morph(interp, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interp, pmc->vtable->base_type);
    }
    VTABLE_morph(interp, dest, pmc->vtable->base_type);
    RE(dest) = RE(pmc) + value;
    IM(dest) = IM(pmc);
    return dest;

}
#line 304 "src/pmc/complex.c"
void
Parrot_Complex_i_add(Interp *interp, PMC* pmc, PMC* value)
#line 692 "./src/pmc/complex.pmc"
{
        RE(pmc) += VTABLE_get_number(interp, value);
}
#line 311 "src/pmc/complex.c"
void
Parrot_Complex_i_add_Complex(Interp *interp, PMC* pmc, PMC* value)
{
        RE(pmc) += RE(value);
        IM(pmc) += IM(value);
}
void
Parrot_Complex_i_add_int(Interp *interp, PMC* pmc, INTVAL value)
#line 702 "./src/pmc/complex.pmc"
{
    RE(pmc) += (FLOATVAL)value;

}
#line 325 "src/pmc/complex.c"
void
Parrot_Complex_i_add_float(Interp *interp, PMC* pmc, FLOATVAL value)
#line 705 "./src/pmc/complex.pmc"
{
    RE(pmc) += value;

}
#line 333 "src/pmc/complex.c"
PMC*
Parrot_Complex_subtract(Interp *interp, PMC* pmc, PMC* value, PMC* dest)
#line 723 "./src/pmc/complex.pmc"
{
        if (dest)
            VTABLE_morph(interp, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interp, pmc->vtable->base_type);
        }
        RE(dest) = RE(pmc) - VTABLE_get_number(interp, value);
        IM(dest) = IM(pmc);
        return dest;
}
#line 347 "src/pmc/complex.c"
PMC*
Parrot_Complex_subtract_Complex(Interp *interp, PMC* pmc, PMC* value, PMC* dest)
{
        if (dest)
            VTABLE_morph(interp, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interp, pmc->vtable->base_type);
        }
        RE(dest) = RE(pmc) - RE(value);
        IM(dest) = IM(pmc) - IM(value);
        return dest;
}
PMC*
Parrot_Complex_subtract_int(Interp *interp, PMC* pmc, INTVAL value, PMC* dest)
#line 746 "./src/pmc/complex.pmc"
{
    if (dest)
        VTABLE_morph(interp, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interp, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) - value;
    IM(dest) = IM(pmc);
    return dest;

}
#line 374 "src/pmc/complex.c"
PMC*
Parrot_Complex_subtract_float(Interp *interp, PMC* pmc, FLOATVAL value, PMC* dest)
#line 757 "./src/pmc/complex.pmc"
{
    if (dest)
        VTABLE_morph(interp, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interp, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) - value;
    IM(dest) = IM(pmc);
    return dest;

}
#line 389 "src/pmc/complex.c"
void
Parrot_Complex_i_subtract(Interp *interp, PMC* pmc, PMC* value)
#line 768 "./src/pmc/complex.pmc"
{
        RE(pmc) -= VTABLE_get_number(interp, value);
}
#line 396 "src/pmc/complex.c"
void
Parrot_Complex_i_subtract_Complex(Interp *interp, PMC* pmc, PMC* value)
{
        RE(pmc) -= RE(value);
        IM(pmc) -= IM(value);
}
void
Parrot_Complex_i_subtract_int(Interp *interp, PMC* pmc, INTVAL value)
#line 778 "./src/pmc/complex.pmc"
{
    RE(pmc) -= (FLOATVAL)value;

}
#line 410 "src/pmc/complex.c"
void
Parrot_Complex_i_subtract_float(Interp *interp, PMC* pmc, FLOATVAL value)
#line 781 "./src/pmc/complex.pmc"
{
    RE(pmc) -= value;

}
#line 418 "src/pmc/complex.c"
PMC*
Parrot_Complex_multiply(Interp *interp, PMC* pmc, PMC* value, PMC* dest)
#line 814 "./src/pmc/complex.pmc"
{
        FLOATVAL re = RE(pmc) * VTABLE_get_number(interp, value);
        FLOATVAL im = IM(pmc) * VTABLE_get_number(interp, value);
        if (dest)
            VTABLE_morph(interp, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interp, pmc->vtable->base_type);
        }
        RE(dest) = re;
        IM(dest) = im;
        return dest;
}
#line 434 "src/pmc/complex.c"
PMC*
Parrot_Complex_multiply_Complex(Interp *interp, PMC* pmc, PMC* value, PMC* dest)
{
        FLOATVAL re = RE(pmc) * RE(value) - IM(pmc) * IM(value);
        FLOATVAL im = IM(pmc) * RE(value) + RE(pmc) * IM(value);
        if (dest)
            VTABLE_morph(interp, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interp, pmc->vtable->base_type);
        }
        RE(dest) = re;
        IM(dest) = im;
        return dest;
}
PMC*
Parrot_Complex_multiply_int(Interp *interp, PMC* pmc, INTVAL value, PMC* dest)
#line 841 "./src/pmc/complex.pmc"
{
    if (dest)
        VTABLE_morph(interp, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interp, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) * value;
    IM(dest) = IM(pmc) * value;
    return dest;

}
#line 463 "src/pmc/complex.c"
PMC*
Parrot_Complex_multiply_float(Interp *interp, PMC* pmc, FLOATVAL value, PMC* dest)
#line 852 "./src/pmc/complex.pmc"
{
    if (dest)
        VTABLE_morph(interp, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interp, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) * value;
    IM(dest) = IM(pmc) * value;
    return dest;

}
#line 478 "src/pmc/complex.c"
void
Parrot_Complex_i_multiply(Interp *interp, PMC* pmc, PMC* value)
#line 863 "./src/pmc/complex.pmc"
{
        RE(pmc) *= VTABLE_get_number(interp, value);
        IM(pmc) *= VTABLE_get_number(interp, value);
}
#line 486 "src/pmc/complex.c"
void
Parrot_Complex_i_multiply_Complex(Interp *interp, PMC* pmc, PMC* value)
{
        FLOATVAL re = RE(pmc) * RE(value) - IM(pmc) * IM(value);
        FLOATVAL im = IM(pmc) * RE(value) + RE(pmc) * IM(value);
        RE(pmc) = re;
        IM(pmc) = im;
}
void
Parrot_Complex_i_multiply_int(Interp *interp, PMC* pmc, INTVAL value)
#line 876 "./src/pmc/complex.pmc"
{
    RE(pmc) *= value;
    IM(pmc) *= value;

}
#line 503 "src/pmc/complex.c"
void
Parrot_Complex_i_multiply_float(Interp *interp, PMC* pmc, FLOATVAL value)
#line 881 "./src/pmc/complex.pmc"
{
    RE(pmc) *= value;
    IM(pmc) *= value;

}
#line 512 "src/pmc/complex.c"
PMC*
Parrot_Complex_divide(Interp *interp, PMC* pmc, PMC* value, PMC* dest)
#line 916 "./src/pmc/complex.pmc"
{
        FLOATVAL d = VTABLE_get_number(interp, value);
        float_check_divide_zero(interp, d);
        if (dest)
            VTABLE_morph(interp, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interp, pmc->vtable->base_type);
        }
        RE(dest) = RE(pmc) / d;
        IM(dest) = IM(pmc) / d;
        return dest;
}
#line 528 "src/pmc/complex.c"
PMC*
Parrot_Complex_divide_Complex(Interp *interp, PMC* pmc, PMC* value, PMC* dest)
{
        FLOATVAL mod, re, im;

        complex_check_divide_zero(interp, value);
        if (dest)
            VTABLE_morph(interp, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interp, pmc->vtable->base_type);
        }
        /* a little speed optimisation: cache an intermediate number;
            I'm not sure the compiler does this */
        mod = (RE(value) * RE(value) + IM(value) * IM(value));
        re = (RE(pmc) * RE(value) + IM(pmc) * IM(value)) / mod;
        im = (IM(pmc) * RE(value) - RE(pmc) * IM(value)) / mod;
        RE(dest) = re;
        IM(dest) = im;
        return dest;
}
PMC*
Parrot_Complex_divide_int(Interp *interp, PMC* pmc, INTVAL value, PMC* dest)
#line 949 "./src/pmc/complex.pmc"
{
    int_check_divide_zero(interp, value);
    if (dest)
        VTABLE_morph(interp, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interp, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) / value;
    IM(dest) = IM(pmc) / value;
    return dest;

}
#line 564 "src/pmc/complex.c"
PMC*
Parrot_Complex_divide_float(Interp *interp, PMC* pmc, FLOATVAL value, PMC* dest)
#line 961 "./src/pmc/complex.pmc"
{
    float_check_divide_zero(interp, value);
    if (dest)
        VTABLE_morph(interp, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interp, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) / value;
    IM(dest) = IM(pmc) / value;
    return dest;

}
#line 580 "src/pmc/complex.c"
void
Parrot_Complex_i_divide(Interp *interp, PMC* pmc, PMC* value)
#line 973 "./src/pmc/complex.pmc"
{
        FLOATVAL d = VTABLE_get_number(interp, value);
        float_check_divide_zero(interp, d);
        RE(pmc) /= d;
        IM(pmc) /= d;
}
#line 590 "src/pmc/complex.c"
void
Parrot_Complex_i_divide_Complex(Interp *interp, PMC* pmc, PMC* value)
{
        FLOATVAL mod, re, im;

        complex_check_divide_zero(interp, value);
        /* a little speed optimisation: cache an intermediate number;
            I'm not sure the compiler does this */
        mod = (RE(value) * RE(value) + IM(value) * IM(value));
        re = (RE(pmc) * RE(value) + IM(pmc) * IM(value)) / mod;
        im = (IM(pmc) * RE(value) - RE(pmc) * IM(value)) / mod;
        RE(pmc) = re;
        IM(pmc) = im;
}
void
Parrot_Complex_i_divide_int(Interp *interp, PMC* pmc, INTVAL value)
#line 994 "./src/pmc/complex.pmc"
{
    int_check_divide_zero(interp, value);
    RE(pmc) /= value;
    IM(pmc) /= value;

}
#line 614 "src/pmc/complex.c"
void
Parrot_Complex_i_divide_float(Interp *interp, PMC* pmc, FLOATVAL value)
#line 1000 "./src/pmc/complex.pmc"
{
    float_check_divide_zero(interp, value);
    RE(pmc) /= value;
    IM(pmc) /= value;

}
#line 624 "src/pmc/complex.c"
PMC*
Parrot_Complex_pow(Interp *interp, PMC* pmc, PMC* value, PMC* dest)
#line 1577 "./src/pmc/complex.pmc"
{
             PMC *l = pmc_new(interp, pmc->vtable->base_type);
             if (dest)
                 VTABLE_morph(interp, dest, pmc->vtable->base_type);
             else
                 dest = pmc_new(interp, pmc->vtable->base_type);
             l = Parrot_Complex_multiply(interp,
                     Parrot_Complex_nci_ln(interp, pmc), value, l);
             dest = Parrot_Complex_nci_exp(interp, l);
             return dest;
}
#line 639 "src/pmc/complex.c"
PMC*
Parrot_Complex_pow_Complex(Interp *interp, PMC* pmc, PMC* value, PMC* dest)
{
             PMC *l = pmc_new(interp, pmc->vtable->base_type);
             if (dest)
                 VTABLE_morph(interp, dest, pmc->vtable->base_type);
             else
                 dest = pmc_new(interp, pmc->vtable->base_type);
             l = Parrot_Complex_multiply_Complex(interp,
                     Parrot_Complex_nci_ln(interp, pmc), value, l);
             dest = Parrot_Complex_nci_exp(interp, l);
             return dest;
}
INTVAL
Parrot_Complex_is_equal(Interp *interp, PMC* pmc, PMC* value)
#line 1044 "./src/pmc/complex.pmc"
{
        if (IM(pmc) != 0.0)
            return (INTVAL)0;
        return (RE(pmc) == VTABLE_get_number(interp, value));
}
#line 661 "src/pmc/complex.c"
INTVAL
Parrot_Complex_is_equal_Complex(Interp *interp, PMC* pmc, PMC* value)
{
        return (INTVAL)(
                RE(pmc) == RE(value) &&
                IM(pmc) == IM(value));
}
PMC*
Parrot_Complex_absolute(Interp *interp, PMC* pmc, PMC *dest)
#line 1081 "./src/pmc/complex.pmc"
{
    FLOATVAL d = sqrt(RE(pmc)*RE(pmc) + IM(pmc)*IM(pmc));
    if (!dest)
        dest = pmc_new(interp,
            Parrot_get_ctx_HLL_type(interp, enum_class_Float));
    VTABLE_set_number_native(interp, dest, d);
    return dest;

}
#line 681 "src/pmc/complex.c"
PMC*
Parrot_Complex_clone(Interp *interp, PMC* pmc)
#line 365 "./src/pmc/complex.pmc"
{
    PMC* dest = pmc_new_noinit(interp, pmc->vtable->base_type);
    FLOATVAL* data = (FLOATVAL*)mem_sys_allocate(2 * sizeof (FLOATVAL));
    PMC_struct_val(dest) = data;
    PObj_active_destroy_SET(dest);
    RE(dest) = RE(pmc);
    IM(dest) = IM(pmc);
    return dest;

}
#line 695 "src/pmc/complex.c"
void
Parrot_Complex_destroy(Interp *interp, PMC* pmc)
#line 354 "./src/pmc/complex.pmc"
{
    mem_sys_free(PMC_struct_val(pmc));
    PMC_struct_val(pmc) = NULL;

}
#line 704 "src/pmc/complex.c"
INTVAL
Parrot_Complex_get_bool(Interp *interp, PMC* pmc)
#line 412 "./src/pmc/complex.pmc"
{
    return (INTVAL)(RE(pmc) != 0.0 || IM(pmc) != 0.0);

}
#line 712 "src/pmc/complex.c"
INTVAL
Parrot_Complex_get_integer(Interp *interp, PMC* pmc)
#line 397 "./src/pmc/complex.pmc"
{
    return (INTVAL)( VTABLE_get_number(interp, pmc));

}
#line 720 "src/pmc/complex.c"
INTVAL
Parrot_Complex_get_integer_keyed(Interp *interp, PMC* pmc, PMC* key)
#line 436 "./src/pmc/complex.pmc"
{
    STRING* s = VTABLE_get_string(interp, key);
    return  VTABLE_get_integer_keyed_str(interp, pmc, s);

}
#line 729 "src/pmc/complex.c"
INTVAL
Parrot_Complex_get_integer_keyed_str(Interp *interp, PMC* pmc, STRING* key)
#line 441 "./src/pmc/complex.pmc"
{
    return (INTVAL)( VTABLE_get_number_keyed_str(interp, pmc, key));

}
#line 737 "src/pmc/complex.c"
FLOATVAL
Parrot_Complex_get_number(Interp *interp, PMC* pmc)
#line 401 "./src/pmc/complex.pmc"
{
    return sqrt(RE(pmc)*RE(pmc) + IM(pmc)*IM(pmc));

}
#line 745 "src/pmc/complex.c"
FLOATVAL
Parrot_Complex_get_number_keyed(Interp *interp, PMC* pmc, PMC* key)
#line 445 "./src/pmc/complex.pmc"
{
    STRING* s = VTABLE_get_string(interp, key);
    return  VTABLE_get_number_keyed_str(interp, pmc, s);

}
#line 754 "src/pmc/complex.c"
FLOATVAL
Parrot_Complex_get_number_keyed_int(Interp *interp, PMC* pmc, INTVAL key)
#line 509 "./src/pmc/complex.pmc"
{
    switch (key) {
        case 0:
            return RE(pmc);
        case 1:
            return IM(pmc);
        default:
            real_exception(interp, NULL, E_KeyError,
                    "Complex: key must be 0 or 1");
    }
    return 0.0;

}
#line 771 "src/pmc/complex.c"
FLOATVAL
Parrot_Complex_get_number_keyed_str(Interp *interp, PMC* pmc, STRING* key)
#line 450 "./src/pmc/complex.pmc"
{
    FLOATVAL *num = complex_locate_keyed_num(interp, pmc, key);
    if (num)
        return *num;
    return 0.0;

}
#line 782 "src/pmc/complex.c"
PMC*
Parrot_Complex_get_pmc_keyed(Interp *interp, PMC* pmc, PMC* key)
#line 457 "./src/pmc/complex.pmc"
{
    STRING* s = VTABLE_get_string(interp, key);
    return  VTABLE_get_pmc_keyed_str(interp, pmc, s);

}
#line 791 "src/pmc/complex.c"
PMC*
Parrot_Complex_get_pmc_keyed_int(Interp *interp, PMC* pmc, INTVAL key)
#line 482 "./src/pmc/complex.pmc"
{
    PMC *ret;
    FLOATVAL val;

    ret = pmc_new(interp, enum_class_Float);
    val =  VTABLE_get_number_keyed_int(interp, pmc, key);
    VTABLE_set_number_native(interp, ret, val);
    return ret;

}
#line 805 "src/pmc/complex.c"
PMC*
Parrot_Complex_get_pmc_keyed_str(Interp *interp, PMC* pmc, STRING* key)
#line 462 "./src/pmc/complex.pmc"
{
    PMC *ret;
    FLOATVAL val;

    ret = pmc_new(interp, enum_class_Float);
    val =  VTABLE_get_number_keyed_str(interp, pmc, key);
    VTABLE_set_number_native(interp, ret, val);
    return ret;

}
#line 819 "src/pmc/complex.c"
STRING*
Parrot_Complex_get_string(Interp *interp, PMC* pmc)
#line 405 "./src/pmc/complex.pmc"
{
    STRING *s;
        s = Parrot_sprintf_c(interp,
                "%vg%+vgi", RE(pmc), IM(pmc));
    return s;

}
#line 830 "src/pmc/complex.c"
void
Parrot_Complex_i_absolute(Interp *interp, PMC* pmc)
#line 1090 "./src/pmc/complex.pmc"
{
    FLOATVAL d = sqrt(RE(pmc)*RE(pmc) + IM(pmc)*IM(pmc));
    VTABLE_morph(interp, pmc, enum_class_Float);   /* XXX */
    VTABLE_set_number_native(interp, pmc, d);

}
#line 840 "src/pmc/complex.c"
void
Parrot_Complex_i_neg(Interp *interp, PMC* pmc)
#line 1029 "./src/pmc/complex.pmc"
{
    RE(pmc) = - RE(pmc);
    IM(pmc) = - IM(pmc);

}
#line 849 "src/pmc/complex.c"
void
Parrot_Complex_init(Interp *interp, PMC* pmc)
#line 341 "./src/pmc/complex.pmc"
{
    /* XXX should check if mem_sys_allocate failed */
    FLOATVAL* data = (FLOATVAL*)mem_sys_allocate(2 * sizeof (FLOATVAL));
    PMC_struct_val(pmc) = data;
    PObj_active_destroy_SET(pmc);
    RE(pmc) = IM(pmc) = 0.0;

}
#line 861 "src/pmc/complex.c"
void
Parrot_Complex_init_pmc(Interp *interp, PMC* pmc, PMC* initializer)
#line 349 "./src/pmc/complex.pmc"
{
    /* XXX not implemented */
     VTABLE_init(interp, pmc);

}
#line 870 "src/pmc/complex.c"
PMC*
Parrot_Complex_instantiate(Interp *interp, PMC* pmc, PMC* sig)
#line 240 "./src/pmc/complex.pmc"
{
    PMC *res = pmc_new(interp, enum_class_Complex);
    FLOATVAL re = 0.0, im = 0.0;
    int argcI = REG_INT(1);
    int argcS = REG_INT(2);
    int argcP = REG_INT(3);
    int argcN = REG_INT(4);

    return PMCNULL;  /* TODO */
    /*
     * we can only allow 0..2 arguments of one kind. For
     * mixed (e.g. N,P) args the order of arguments isn't fixed
     */
    if (argcP) {
        re = VTABLE_get_number(interp, REG_PMC(5));
        if (argcP == 2)
            im = VTABLE_get_number(interp, REG_PMC(6));
        /*
         * TODO throw exception if argument mismatch
         */
    }
    else if (argcN) {
        re = REG_NUM(5);
        if (argcN == 2)
            im = REG_NUM(6);
    }
    else if (argcI) {
        re = REG_INT(5);
        if (argcI == 2)
            im = REG_INT(6);
    }
    else if (argcS == 1) {
        complex_parse_string(interp, &RE(res), &IM(res), REG_STR(5));
        return res;
    }
    RE(res) = re;
    IM(res) = im;
    return res;

}
#line 914 "src/pmc/complex.c"
opcode_t*
Parrot_Complex_invoke(Interp *interp, PMC* pmc, void *next)
#line 290 "./src/pmc/complex.pmc"
{
    int argcP = REG_INT(3);
    PMC *res  = pmc_new(interp, enum_class_Complex);

    if (argcP == 1) {
        PMC *arg;
        arg = REG_PMC(5);

        if (arg->vtable->base_type == enum_class_String)
            VTABLE_set_string_native(interp, res, PMC_str_val(arg));
        else
            RE(res) = VTABLE_get_number(interp, arg);
    }
    else if (argcP == 2) {
        RE(res) = VTABLE_get_number(interp, REG_PMC(5));
        IM(res) = VTABLE_get_number(interp, REG_PMC(6));
    }

    REG_PMC(5) = res;
    return (opcode_t *)next;

}
#line 940 "src/pmc/complex.c"
void
Parrot_Complex_morph(Interp *interp, PMC* pmc, INTVAL type)
#line 359 "./src/pmc/complex.pmc"
{
    if (pmc->vtable->base_type == type)
        return;
    Parrot_default_morph(interp, pmc, type);

}
#line 950 "src/pmc/complex.c"
PMC*
Parrot_Complex_neg(Interp *interp, PMC* pmc, PMC* dest)
#line 1019 "./src/pmc/complex.pmc"
{
    if (!dest)
        dest = pmc_new(interp, pmc->vtable->base_type);
    else
        VTABLE_morph(interp, dest, pmc->vtable->base_type);
    RE(dest) = - RE(pmc);
    IM(dest) = - IM(pmc);
    return dest;

}
#line 964 "src/pmc/complex.c"
void
Parrot_Complex_set_integer_keyed(Interp *interp, PMC* pmc, PMC* key, INTVAL value)
#line 603 "./src/pmc/complex.pmc"
{
     VTABLE_set_number_keyed(interp, pmc, key, value);

}
#line 972 "src/pmc/complex.c"
void
Parrot_Complex_set_integer_keyed_str(Interp *interp, PMC* pmc, STRING* key, INTVAL value)
#line 607 "./src/pmc/complex.pmc"
{
     VTABLE_set_number_keyed_str(interp, pmc, key, value);

}
#line 980 "src/pmc/complex.c"
void
Parrot_Complex_set_integer_native(Interp *interp, PMC* pmc, INTVAL value)
#line 573 "./src/pmc/complex.pmc"
{
     VTABLE_set_number_native(interp, pmc, value);

}
#line 988 "src/pmc/complex.c"
void
Parrot_Complex_set_number_keyed(Interp *interp, PMC* pmc, PMC* key, FLOATVAL value)
#line 611 "./src/pmc/complex.pmc"
{
    STRING* s = VTABLE_get_string(interp, key);
     VTABLE_set_number_keyed_str(interp, pmc, s, value);

}
#line 997 "src/pmc/complex.c"
void
Parrot_Complex_set_number_keyed_int(Interp *interp, PMC* pmc, INTVAL key, FLOATVAL v)
#line 522 "./src/pmc/complex.pmc"
{
    switch (key) {
        case 0:
            RE(pmc) = v;
            break;
        case 1:
            IM(pmc) = v;
            break;
        default:
            real_exception(interp, NULL, E_KeyError,
                    "Complex: key must be 0 or 1");
    }

}
#line 1015 "src/pmc/complex.c"
void
Parrot_Complex_set_number_keyed_str(Interp *interp, PMC* pmc, STRING* key, FLOATVAL value)
#line 616 "./src/pmc/complex.pmc"
{
    FLOATVAL *num = complex_locate_keyed_num(interp, pmc, key);
    if (num)
        *num = value;

}
#line 1025 "src/pmc/complex.c"
void
Parrot_Complex_set_number_native(Interp *interp, PMC* pmc, FLOATVAL value)
#line 577 "./src/pmc/complex.pmc"
{
    RE(pmc) = value;
    IM(pmc) = 0.0;

}
#line 1034 "src/pmc/complex.c"
void
Parrot_Complex_set_pmc(Interp *interp, PMC* pmc, PMC* value)
#line 563 "./src/pmc/complex.pmc"
{
    if (value->vtable->base_type == enum_class_Complex) {
        RE(pmc) = RE(value);
        IM(pmc) = IM(value);
    }
    else {
         VTABLE_set_string_native(interp, pmc, VTABLE_get_string(interp, value));
    }

}
#line 1048 "src/pmc/complex.c"
void
Parrot_Complex_set_pmc_keyed(Interp *interp, PMC* pmc, PMC* key, PMC* value)
#line 622 "./src/pmc/complex.pmc"
{
    FLOATVAL f = VTABLE_get_number(interp, value);
     VTABLE_set_number_keyed(interp, pmc, key, f);

}
#line 1057 "src/pmc/complex.c"
void
Parrot_Complex_set_pmc_keyed_str(Interp *interp, PMC* pmc, STRING* key, PMC* value)
#line 627 "./src/pmc/complex.pmc"
{
    FLOATVAL f = VTABLE_get_number(interp, value);
     VTABLE_set_number_keyed_str(interp, pmc, key, f);

}
#line 1066 "src/pmc/complex.c"
void
Parrot_Complex_set_string_native(Interp *interp, PMC* pmc, STRING* value)
#line 559 "./src/pmc/complex.pmc"
{
    complex_parse_string(interp, &RE(pmc), &IM(pmc), value);

}
#line 1074 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_ln(Interp *interp, PMC* pmc)
#line 1117 "./src/pmc/complex.pmc"
{
    FLOATVAL re, im;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    re = RE(pmc);
    im = IM(pmc);
    /* This is necessary for atan2 to behave */
    if (im == -0.0) im = 0.0;
    RE(d) = log(sqrt(re*re + im*im));
    IM(d) = atan2(im, re);
    return d;

}
#line 1090 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_exp(Interp *interp, PMC* pmc)
#line 1141 "./src/pmc/complex.pmc"
{
    FLOATVAL re, im, f;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    re = RE(pmc);
    im = IM(pmc);
    f = exp(re);
    RE(d) = f * cos(im);
    /* If only sin(pi) worked. */
    if (im == 4.0 * atan(1.0))
        IM(d) = 0.0;
    else
        IM(d) = f * sin(im);
    return d;

}
#line 1109 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_sin(Interp *interp, PMC* pmc)
#line 1189 "./src/pmc/complex.pmc"
{
    FLOATVAL re, im;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    re = RE(pmc);
    im = IM(pmc);
    if (im == 0.0) {
        RE(d) = sin(re);
        IM(d) = 0.0;
        return d;
    }
    if (re == 0.0) {
        RE(d) = 0.0;
        IM(d) = sinh(im);
        return d;
    }
    RE(d) = sin(re) * cosh(im);
    if (im == 0.0)
        IM(d) = im;
    else
        IM(d) = cos(re) * sinh(im);
    return d;

}
#line 1136 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_cos(Interp *interp, PMC* pmc)
#line 1212 "./src/pmc/complex.pmc"
{
    FLOATVAL re, im;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    re = RE(pmc);
    im = IM(pmc);
    if (re == 0.0) {
        RE(d) = cosh(im);
        IM(d) = 0.0;
        return d;
    }
    if (im == 0.0) {
        RE(d) = cos(re);
        IM(d) = 0.0;
        return d;
    }
    RE(d) = cos(re) * cosh(im);
    IM(d) = -1.0 * sin(re) * sinh(im);
    return d;

}
#line 1160 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_tan(Interp *interp, PMC* pmc)
#line 1232 "./src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    PMC *e = pmc_new(interp, pmc->vtable->base_type);
    d = Parrot_Complex_nci_sin(interp, pmc);
    e = Parrot_Complex_nci_cos(interp, pmc);
    Parrot_Complex_i_divide_Complex(interp, d, e);
    return d;

}
#line 1173 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_cot(Interp *interp, PMC* pmc)
#line 1241 "./src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    PMC *e = pmc_new(interp, pmc->vtable->base_type);
    d = Parrot_Complex_nci_cos(interp, pmc);
    e = Parrot_Complex_nci_sin(interp, pmc);
    Parrot_Complex_i_divide_Complex(interp, d, e);
    return d;

}
#line 1186 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_sec(Interp *interp, PMC* pmc)
#line 1250 "./src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    PMC *e = pmc_new(interp, pmc->vtable->base_type);
    RE(d) = 1.0;
    IM(d) = 0.0;
    e = Parrot_Complex_nci_cos(interp, pmc);
    Parrot_Complex_i_divide_Complex(interp, d, e);
    return d;

}
#line 1200 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_csc(Interp *interp, PMC* pmc)
#line 1260 "./src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    PMC *e = pmc_new(interp, pmc->vtable->base_type);
    RE(d) = 1.0;
    IM(d) = 0.0;
    e = Parrot_Complex_nci_sin(interp, pmc);
    Parrot_Complex_i_divide_Complex(interp, d, e);
    return d;

}
#line 1214 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_asin(Interp *interp, PMC* pmc)
#line 1303 "./src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    PMC *e = pmc_new(interp, pmc->vtable->base_type);
    e = Parrot_Complex_multiply_Complex(interp, pmc, pmc, e);
    RE(e) = 1.0 - RE(e);
    IM(e) = -IM(e);
    d = Parrot_Complex_nci_sqrt(interp, e);
    RE(d) -= IM(pmc);
    IM(d) += RE(pmc);
    d = Parrot_Complex_nci_ln(interp, d);
    RE(e) = IM(d);
    IM(e) = -RE(d);
    return e;

}
#line 1233 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_acos(Interp *interp, PMC* pmc)
#line 1318 "./src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    PMC *e = pmc_new(interp, pmc->vtable->base_type);
    e = Parrot_Complex_multiply_Complex(interp, pmc, pmc, e);
    RE(e) = 1.0 - RE(e);
    IM(e) = -IM(e);
    d = Parrot_Complex_nci_sqrt(interp, e);
    RE(d) += IM(pmc);
    IM(d) -= RE(pmc);
    e = Parrot_Complex_nci_ln(interp, d);
    RE(d) = IM(e) + 2.0 * atan(1);
    IM(d) = -RE(e);
    return d;

}
#line 1252 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_atan(Interp *interp, PMC* pmc)
#line 1333 "./src/pmc/complex.pmc"
{
    FLOATVAL re, im;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    PMC *e = pmc_new(interp, pmc->vtable->base_type);
    re = RE(pmc);
    im = IM(pmc);
    RE(d) = re;
    IM(d) = 1 + im;
    RE(e) = -re;
    IM(e) = 1 - im;
    Parrot_Complex_i_divide_Complex(interp, d, e);
    d = Parrot_Complex_nci_ln(interp, d);
    RE(e) = IM(d) / -2.0;
    IM(e) = RE(d) / 2.0;
    return e;

}
#line 1273 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_acot(Interp *interp, PMC* pmc)
#line 1351 "./src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    d = Parrot_Complex_nci_atan(interp, d);
    return d;

}
#line 1288 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_acsc(Interp *interp, PMC* pmc)
#line 1362 "./src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    d = Parrot_Complex_nci_asin(interp, d);
    return d;

}
#line 1303 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_asec(Interp *interp, PMC* pmc)
#line 1373 "./src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    d = Parrot_Complex_nci_acos(interp, d);
    return d;

}
#line 1318 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_sinh(Interp *interp, PMC* pmc)
#line 1404 "./src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = sinh(a) * cos(b);
    IM(d) = cosh(a) * sin(b);
    return d;

}
#line 1332 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_cosh(Interp *interp, PMC* pmc)
#line 1414 "./src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = cosh(a) * cos(b);
    IM(d) = sinh(a) * sin(b);
    return d;

}
#line 1346 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_tanh(Interp *interp, PMC* pmc)
#line 1424 "./src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    PMC *e = pmc_new(interp, pmc->vtable->base_type);
    d = Parrot_Complex_nci_sinh(interp, pmc);
    e = Parrot_Complex_nci_cosh(interp, pmc);
    Parrot_Complex_i_divide_Complex(interp, d, e);
    return d;

}
#line 1359 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_coth(Interp *interp, PMC* pmc)
#line 1433 "./src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    d = Parrot_Complex_nci_tanh(interp, pmc);
    a = RE(d);
    b = IM(d);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    return d;

}
#line 1374 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_csch(Interp *interp, PMC* pmc)
#line 1444 "./src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    d = Parrot_Complex_nci_sinh(interp, pmc);
    a = RE(d);
    b = IM(d);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    return d;

}
#line 1389 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_sech(Interp *interp, PMC* pmc)
#line 1455 "./src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    d = Parrot_Complex_nci_cosh(interp, pmc);
    a = RE(d);
    b = IM(d);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    return d;

}
#line 1404 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_asinh(Interp *interp, PMC* pmc)
#line 1494 "./src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    PMC *e = pmc_new(interp, pmc->vtable->base_type);
    RE(d) = IM(pmc);
    IM(d) = -RE(pmc);
    d = Parrot_Complex_nci_asin(interp, d);
    RE(e) = -IM(d);
    IM(e) = RE(d);
    return e;

}
#line 1419 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_acosh(Interp *interp, PMC* pmc)
#line 1505 "./src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    PMC *e = pmc_new(interp, pmc->vtable->base_type);
    d = Parrot_Complex_nci_acos(interp, pmc);
    RE(e) = -IM(d);
    IM(e) = RE(d);
    return e;

}
#line 1432 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_atanh(Interp *interp, PMC* pmc)
#line 1514 "./src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    PMC *e = pmc_new(interp, pmc->vtable->base_type);
    RE(d) = IM(pmc);
    IM(d) = -RE(pmc);
    d = Parrot_Complex_nci_atan(interp, d);
    RE(e) = -IM(d);
    IM(e) = RE(d);
    return e;

}
#line 1447 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_acoth(Interp *interp, PMC* pmc)
#line 1525 "./src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    d = Parrot_Complex_nci_atanh(interp, d);
    return d;

}
#line 1462 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_acsch(Interp *interp, PMC* pmc)
#line 1536 "./src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    d = Parrot_Complex_nci_asinh(interp, d);
    return d;

}
#line 1477 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_asech(Interp *interp, PMC* pmc)
#line 1547 "./src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    d = Parrot_Complex_nci_acosh(interp, d);
    return d;

}
#line 1492 "src/pmc/complex.c"
PMC*
Parrot_Complex_nci_sqrt(Interp *interp, PMC* pmc)
#line 1603 "./src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interp, pmc->vtable->base_type);
    d = Parrot_Complex_nci_ln(interp, pmc);
    RE(d) *= 0.5;
    IM(d) *= 0.5;
    d = Parrot_Complex_nci_exp(interp, d);
    return d;

}
#line 1505 "src/pmc/complex.c"
void
Parrot_Complex_ro_add_attribute(Interp *interp, PMC* pmc, STRING* name, PMC* type)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "add_attribute() in read-only instance of Complex");
}
void
Parrot_Complex_ro_add_method(Interp *interp, PMC* pmc, STRING* method_name, PMC* sub_pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "add_method() in read-only instance of Complex");
}
void
Parrot_Complex_ro_add_parent(Interp *interp, PMC* pmc, PMC* parent)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "add_parent() in read-only instance of Complex");
}
void
Parrot_Complex_ro_add_role(Interp *interp, PMC* pmc, PMC* role)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "add_role() in read-only instance of Complex");
}
void
Parrot_Complex_ro_assign_pmc(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "assign_pmc() in read-only instance of Complex");
}
void
Parrot_Complex_ro_assign_string_native(Interp *interp, PMC* pmc, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "assign_string_native() in read-only instance of Complex");
}
void
Parrot_Complex_ro_decrement(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "decrement() in read-only instance of Complex");
}
void
Parrot_Complex_ro_delete_keyed(Interp *interp, PMC* pmc, PMC* key)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "delete_keyed() in read-only instance of Complex");
}
void
Parrot_Complex_ro_delete_keyed_int(Interp *interp, PMC* pmc, INTVAL key)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "delete_keyed_int() in read-only instance of Complex");
}
void
Parrot_Complex_ro_delete_keyed_str(Interp *interp, PMC* pmc, STRING* key)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "delete_keyed_str() in read-only instance of Complex");
}
PMC*
Parrot_Complex_ro_find_method(Interp *interp, PMC* pmc, STRING* method_name)
 {
    PMC *const method = Parrot_default_find_method(interp, pmc, method_name);
    if (!PMC_IS_NULL(VTABLE_getprop(interp, method, const_string(interp, "write"))))
        return PMCNULL;
    else
        return method;
}
void
Parrot_Complex_ro_i_absolute(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "i_absolute() in read-only instance of Complex");
}
void
Parrot_Complex_ro_i_bitwise_not(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "i_bitwise_not() in read-only instance of Complex");
}
void
Parrot_Complex_ro_i_bitwise_nots(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "i_bitwise_nots() in read-only instance of Complex");
}
void
Parrot_Complex_ro_i_logical_not(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "i_logical_not() in read-only instance of Complex");
}
void
Parrot_Complex_ro_i_neg(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "i_neg() in read-only instance of Complex");
}
void
Parrot_Complex_ro_increment(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "increment() in read-only instance of Complex");
}
void
Parrot_Complex_ro_morph(Interp *interp, PMC* pmc, INTVAL type)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "morph() in read-only instance of Complex");
}
FLOATVAL
Parrot_Complex_ro_pop_float(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "pop_float() in read-only instance of Complex");
    return (FLOATVAL)0;
}
INTVAL
Parrot_Complex_ro_pop_integer(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "pop_integer() in read-only instance of Complex");
    return (INTVAL)0;
}
PMC*
Parrot_Complex_ro_pop_pmc(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "pop_pmc() in read-only instance of Complex");
    return PMCNULL;
}
STRING*
Parrot_Complex_ro_pop_string(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "pop_string() in read-only instance of Complex");
    return (STRING*)0;
}
void
Parrot_Complex_ro_push_float(Interp *interp, PMC* pmc, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "push_float() in read-only instance of Complex");
}
void
Parrot_Complex_ro_push_integer(Interp *interp, PMC* pmc, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "push_integer() in read-only instance of Complex");
}
void
Parrot_Complex_ro_push_pmc(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "push_pmc() in read-only instance of Complex");
}
void
Parrot_Complex_ro_push_string(Interp *interp, PMC* pmc, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "push_string() in read-only instance of Complex");
}
void
Parrot_Complex_ro_remove_attribute(Interp *interp, PMC* pmc, STRING* name)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "remove_attribute() in read-only instance of Complex");
}
void
Parrot_Complex_ro_remove_method(Interp *interp, PMC* pmc, STRING* method_name)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "remove_method() in read-only instance of Complex");
}
void
Parrot_Complex_ro_remove_parent(Interp *interp, PMC* pmc, PMC* parent)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "remove_parent() in read-only instance of Complex");
}
void
Parrot_Complex_ro_remove_role(Interp *interp, PMC* pmc, PMC* role)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "remove_role() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_attr(Interp *interp, PMC* pmc, INTVAL idx, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_attr() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_attr_str(Interp *interp, PMC* pmc, STRING* idx, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_attr_str() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_bignum_int(Interp *interp, PMC* pmc, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_bignum_int() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_bignum_num(Interp *interp, PMC* pmc, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_bignum_num() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_bignum_str(Interp *interp, PMC* pmc, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_bignum_str() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_bool(Interp *interp, PMC* pmc, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_bool() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_integer_keyed(Interp *interp, PMC* pmc, PMC* key, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_integer_keyed() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_integer_keyed_int(Interp *interp, PMC* pmc, INTVAL key, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_integer_keyed_int() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_integer_keyed_str(Interp *interp, PMC* pmc, STRING* key, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_integer_keyed_str() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_integer_native(Interp *interp, PMC* pmc, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_integer_native() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_integer_same(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_integer_same() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_number_keyed(Interp *interp, PMC* pmc, PMC* key, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_number_keyed() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_number_keyed_int(Interp *interp, PMC* pmc, INTVAL key, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_number_keyed_int() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_number_keyed_str(Interp *interp, PMC* pmc, STRING* key, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_number_keyed_str() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_number_native(Interp *interp, PMC* pmc, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_number_native() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_number_same(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_number_same() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_pmc(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pmc() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_pmc_keyed(Interp *interp, PMC* pmc, PMC* key, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pmc_keyed() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_pmc_keyed_int(Interp *interp, PMC* pmc, INTVAL key, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pmc_keyed_int() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_pmc_keyed_str(Interp *interp, PMC* pmc, STRING* key, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pmc_keyed_str() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_pointer(Interp *interp, PMC* pmc, void* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pointer() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_pointer_keyed(Interp *interp, PMC* pmc, PMC* key, void* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pointer_keyed() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_pointer_keyed_int(Interp *interp, PMC* pmc, INTVAL key, void* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pointer_keyed_int() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_pointer_keyed_str(Interp *interp, PMC* pmc, STRING* key, void* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pointer_keyed_str() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_string_keyed(Interp *interp, PMC* pmc, PMC* key, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_string_keyed() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_string_keyed_int(Interp *interp, PMC* pmc, INTVAL key, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_string_keyed_int() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_string_keyed_str(Interp *interp, PMC* pmc, STRING* key, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_string_keyed_str() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_string_native(Interp *interp, PMC* pmc, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_string_native() in read-only instance of Complex");
}
void
Parrot_Complex_ro_set_string_same(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_string_same() in read-only instance of Complex");
}
FLOATVAL
Parrot_Complex_ro_shift_float(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "shift_float() in read-only instance of Complex");
    return (FLOATVAL)0;
}
INTVAL
Parrot_Complex_ro_shift_integer(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "shift_integer() in read-only instance of Complex");
    return (INTVAL)0;
}
PMC*
Parrot_Complex_ro_shift_pmc(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "shift_pmc() in read-only instance of Complex");
    return PMCNULL;
}
STRING*
Parrot_Complex_ro_shift_string(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "shift_string() in read-only instance of Complex");
    return (STRING*)0;
}
void
Parrot_Complex_ro_splice(Interp *interp, PMC* pmc, PMC* value, INTVAL offset, INTVAL count)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "splice() in read-only instance of Complex");
}
void
Parrot_Complex_ro_thaw(Interp *interp, PMC* pmc, visit_info* info)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "thaw() in read-only instance of Complex");
}
void
Parrot_Complex_ro_thawfinish(Interp *interp, PMC* pmc, visit_info* info)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "thawfinish() in read-only instance of Complex");
}
void
Parrot_Complex_ro_unshift_float(Interp *interp, PMC* pmc, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "unshift_float() in read-only instance of Complex");
}
void
Parrot_Complex_ro_unshift_integer(Interp *interp, PMC* pmc, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "unshift_integer() in read-only instance of Complex");
}
void
Parrot_Complex_ro_unshift_pmc(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "unshift_pmc() in read-only instance of Complex");
}
void
Parrot_Complex_ro_unshift_string(Interp *interp, PMC* pmc, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "unshift_string() in read-only instance of Complex");
}
void
Parrot_Complex_class_init(Parrot_Interp interp, int entry, int pass)
{
    const VTABLE temp_base_vtable = {
        NULL, /* namespace */
        enum_class_Complex, /* base_type */
        NULL, /* whoami */
        0|VTABLE_HAS_READONLY_FLAG, /* flags */
        NULL,   /* does_str */
        NULL,   /* isa_str */
        NULL,   /* class */
        NULL,   /* mro */
        NULL,   /* ro_variant_vtable */
        Parrot_Complex_absolute,
        Parrot_default_add_attribute,
        Parrot_default_add_method,
        Parrot_default_add_parent,
        Parrot_default_add_role,
        Parrot_default_assign_pmc,
        Parrot_default_assign_string_native,
        Parrot_default_bitwise_not,
        Parrot_default_bitwise_nots,
        Parrot_default_can,
        Parrot_default_class_type,
        Parrot_Complex_clone,
        Parrot_default_clone_pmc,
        Parrot_default_decrement,
        Parrot_default_defined,
        Parrot_default_defined_keyed,
        Parrot_default_defined_keyed_int,
        Parrot_default_defined_keyed_str,
        Parrot_default_delete_keyed,
        Parrot_default_delete_keyed_int,
        Parrot_default_delete_keyed_str,
        Parrot_default_delprop,
        Parrot_Complex_destroy,
        Parrot_default_does,
        Parrot_default_elements,
        Parrot_default_exists_keyed,
        Parrot_default_exists_keyed_int,
        Parrot_default_exists_keyed_str,
        Parrot_default_find_method,
        Parrot_default_freeze,
        Parrot_default_get_attr,
        Parrot_default_get_attr_str,
        Parrot_default_get_bignum,
        Parrot_Complex_get_bool,
        Parrot_default_get_class,
        Parrot_Complex_get_integer,
        Parrot_Complex_get_integer_keyed,
        Parrot_default_get_integer_keyed_int,
        Parrot_Complex_get_integer_keyed_str,
        Parrot_default_get_iter,
        Parrot_Complex_get_number,
        Parrot_Complex_get_number_keyed,
        Parrot_Complex_get_number_keyed_int,
        Parrot_Complex_get_number_keyed_str,
        Parrot_default_get_pmc,
        Parrot_Complex_get_pmc_keyed,
        Parrot_Complex_get_pmc_keyed_int,
        Parrot_Complex_get_pmc_keyed_str,
        Parrot_default_get_pointer,
        Parrot_default_get_pointer_keyed,
        Parrot_default_get_pointer_keyed_int,
        Parrot_default_get_pointer_keyed_str,
        Parrot_default_get_repr,
        Parrot_Complex_get_string,
        Parrot_default_get_string_keyed,
        Parrot_default_get_string_keyed_int,
        Parrot_default_get_string_keyed_str,
        Parrot_default_getprop,
        Parrot_default_getprops,
        Parrot_default_hash,
        Parrot_Complex_i_absolute,
        Parrot_default_i_bitwise_not,
        Parrot_default_i_bitwise_nots,
        Parrot_default_i_logical_not,
        Parrot_Complex_i_neg,
        Parrot_default_increment,
        Parrot_Complex_init,
        Parrot_Complex_init_pmc,
        Parrot_default_inspect,
        Parrot_default_inspect_str,
        Parrot_Complex_instantiate,
        Parrot_Complex_invoke,
        Parrot_default_is_same,
        Parrot_default_isa,
        Parrot_default_logical_not,
        Parrot_default_mark,
        Parrot_Complex_morph,
        Parrot_default_name,
        Parrot_Complex_neg,
        Parrot_default_new_from_string,
        Parrot_default_nextkey_keyed,
        Parrot_default_nextkey_keyed_int,
        Parrot_default_nextkey_keyed_str,
        Parrot_default_pmc_namespace,
        Parrot_default_pop_float,
        Parrot_default_pop_integer,
        Parrot_default_pop_pmc,
        Parrot_default_pop_string,
        Parrot_default_push_float,
        Parrot_default_push_integer,
        Parrot_default_push_pmc,
        Parrot_default_push_string,
        Parrot_default_remove_attribute,
        Parrot_default_remove_method,
        Parrot_default_remove_parent,
        Parrot_default_remove_role,
        Parrot_default_set_attr,
        Parrot_default_set_attr_str,
        Parrot_default_set_bignum_int,
        Parrot_default_set_bignum_num,
        Parrot_default_set_bignum_str,
        Parrot_default_set_bool,
        Parrot_Complex_set_integer_keyed,
        Parrot_default_set_integer_keyed_int,
        Parrot_Complex_set_integer_keyed_str,
        Parrot_Complex_set_integer_native,
        Parrot_default_set_integer_same,
        Parrot_Complex_set_number_keyed,
        Parrot_Complex_set_number_keyed_int,
        Parrot_Complex_set_number_keyed_str,
        Parrot_Complex_set_number_native,
        Parrot_default_set_number_same,
        Parrot_Complex_set_pmc,
        Parrot_Complex_set_pmc_keyed,
        Parrot_default_set_pmc_keyed_int,
        Parrot_Complex_set_pmc_keyed_str,
        Parrot_default_set_pointer,
        Parrot_default_set_pointer_keyed,
        Parrot_default_set_pointer_keyed_int,
        Parrot_default_set_pointer_keyed_str,
        Parrot_default_set_string_keyed,
        Parrot_default_set_string_keyed_int,
        Parrot_default_set_string_keyed_str,
        Parrot_Complex_set_string_native,
        Parrot_default_set_string_same,
        Parrot_default_setprop,
        Parrot_default_share,
        Parrot_default_share_ro,
        Parrot_default_shift_float,
        Parrot_default_shift_integer,
        Parrot_default_shift_pmc,
        Parrot_default_shift_string,
        Parrot_default_slice,
        Parrot_default_splice,
        Parrot_default_subclass,
        Parrot_default_substr,
        Parrot_default_substr_str,
        Parrot_default_subtype,
        Parrot_default_thaw,
        Parrot_default_thawfinish,
        Parrot_default_type,
        Parrot_default_type_keyed,
        Parrot_default_type_keyed_int,
        Parrot_default_type_keyed_str,
        Parrot_default_unshift_float,
        Parrot_default_unshift_integer,
        Parrot_default_unshift_pmc,
        Parrot_default_unshift_string,
        Parrot_default_visit
    };

    const VTABLE temp_ro_vtable = {
        NULL, /* namespace */
        enum_class_Complex, /* base_type */
        NULL, /* whoami */
        0|VTABLE_IS_READONLY_FLAG, /* flags */
        NULL,   /* does_str */
        NULL,   /* isa_str */
        NULL,   /* class */
        NULL,   /* mro */
        NULL,   /* ro_variant_vtable */
        Parrot_Complex_absolute,
        Parrot_Complex_ro_add_attribute,
        Parrot_Complex_ro_add_method,
        Parrot_Complex_ro_add_parent,
        Parrot_Complex_ro_add_role,
        Parrot_Complex_ro_assign_pmc,
        Parrot_Complex_ro_assign_string_native,
        Parrot_default_bitwise_not,
        Parrot_default_bitwise_nots,
        Parrot_default_can,
        Parrot_default_class_type,
        Parrot_Complex_clone,
        Parrot_default_clone_pmc,
        Parrot_Complex_ro_decrement,
        Parrot_default_defined,
        Parrot_default_defined_keyed,
        Parrot_default_defined_keyed_int,
        Parrot_default_defined_keyed_str,
        Parrot_Complex_ro_delete_keyed,
        Parrot_Complex_ro_delete_keyed_int,
        Parrot_Complex_ro_delete_keyed_str,
        Parrot_default_delprop,
        Parrot_Complex_destroy,
        Parrot_default_does,
        Parrot_default_elements,
        Parrot_default_exists_keyed,
        Parrot_default_exists_keyed_int,
        Parrot_default_exists_keyed_str,
        Parrot_Complex_ro_find_method,
        Parrot_default_freeze,
        Parrot_default_get_attr,
        Parrot_default_get_attr_str,
        Parrot_default_get_bignum,
        Parrot_Complex_get_bool,
        Parrot_default_get_class,
        Parrot_Complex_get_integer,
        Parrot_Complex_get_integer_keyed,
        Parrot_default_get_integer_keyed_int,
        Parrot_Complex_get_integer_keyed_str,
        Parrot_default_get_iter,
        Parrot_Complex_get_number,
        Parrot_Complex_get_number_keyed,
        Parrot_Complex_get_number_keyed_int,
        Parrot_Complex_get_number_keyed_str,
        Parrot_default_get_pmc,
        Parrot_Complex_get_pmc_keyed,
        Parrot_Complex_get_pmc_keyed_int,
        Parrot_Complex_get_pmc_keyed_str,
        Parrot_default_get_pointer,
        Parrot_default_get_pointer_keyed,
        Parrot_default_get_pointer_keyed_int,
        Parrot_default_get_pointer_keyed_str,
        Parrot_default_get_repr,
        Parrot_Complex_get_string,
        Parrot_default_get_string_keyed,
        Parrot_default_get_string_keyed_int,
        Parrot_default_get_string_keyed_str,
        Parrot_default_getprop,
        Parrot_default_getprops,
        Parrot_default_hash,
        Parrot_Complex_ro_i_absolute,
        Parrot_Complex_ro_i_bitwise_not,
        Parrot_Complex_ro_i_bitwise_nots,
        Parrot_Complex_ro_i_logical_not,
        Parrot_Complex_ro_i_neg,
        Parrot_Complex_ro_increment,
        Parrot_Complex_init,
        Parrot_Complex_init_pmc,
        Parrot_default_inspect,
        Parrot_default_inspect_str,
        Parrot_Complex_instantiate,
        Parrot_Complex_invoke,
        Parrot_default_is_same,
        Parrot_default_isa,
        Parrot_default_logical_not,
        Parrot_default_mark,
        Parrot_Complex_ro_morph,
        Parrot_default_name,
        Parrot_Complex_neg,
        Parrot_default_new_from_string,
        Parrot_default_nextkey_keyed,
        Parrot_default_nextkey_keyed_int,
        Parrot_default_nextkey_keyed_str,
        Parrot_default_pmc_namespace,
        Parrot_Complex_ro_pop_float,
        Parrot_Complex_ro_pop_integer,
        Parrot_Complex_ro_pop_pmc,
        Parrot_Complex_ro_pop_string,
        Parrot_Complex_ro_push_float,
        Parrot_Complex_ro_push_integer,
        Parrot_Complex_ro_push_pmc,
        Parrot_Complex_ro_push_string,
        Parrot_Complex_ro_remove_attribute,
        Parrot_Complex_ro_remove_method,
        Parrot_Complex_ro_remove_parent,
        Parrot_Complex_ro_remove_role,
        Parrot_Complex_ro_set_attr,
        Parrot_Complex_ro_set_attr_str,
        Parrot_Complex_ro_set_bignum_int,
        Parrot_Complex_ro_set_bignum_num,
        Parrot_Complex_ro_set_bignum_str,
        Parrot_Complex_ro_set_bool,
        Parrot_Complex_ro_set_integer_keyed,
        Parrot_Complex_ro_set_integer_keyed_int,
        Parrot_Complex_ro_set_integer_keyed_str,
        Parrot_Complex_ro_set_integer_native,
        Parrot_Complex_ro_set_integer_same,
        Parrot_Complex_ro_set_number_keyed,
        Parrot_Complex_ro_set_number_keyed_int,
        Parrot_Complex_ro_set_number_keyed_str,
        Parrot_Complex_ro_set_number_native,
        Parrot_Complex_ro_set_number_same,
        Parrot_Complex_ro_set_pmc,
        Parrot_Complex_ro_set_pmc_keyed,
        Parrot_Complex_ro_set_pmc_keyed_int,
        Parrot_Complex_ro_set_pmc_keyed_str,
        Parrot_Complex_ro_set_pointer,
        Parrot_Complex_ro_set_pointer_keyed,
        Parrot_Complex_ro_set_pointer_keyed_int,
        Parrot_Complex_ro_set_pointer_keyed_str,
        Parrot_Complex_ro_set_string_keyed,
        Parrot_Complex_ro_set_string_keyed_int,
        Parrot_Complex_ro_set_string_keyed_str,
        Parrot_Complex_ro_set_string_native,
        Parrot_Complex_ro_set_string_same,
        Parrot_default_setprop,
        Parrot_default_share,
        Parrot_default_share_ro,
        Parrot_Complex_ro_shift_float,
        Parrot_Complex_ro_shift_integer,
        Parrot_Complex_ro_shift_pmc,
        Parrot_Complex_ro_shift_string,
        Parrot_default_slice,
        Parrot_Complex_ro_splice,
        Parrot_default_subclass,
        Parrot_default_substr,
        Parrot_default_substr_str,
        Parrot_default_subtype,
        Parrot_Complex_ro_thaw,
        Parrot_Complex_ro_thawfinish,
        Parrot_default_type,
        Parrot_default_type_keyed,
        Parrot_default_type_keyed_int,
        Parrot_default_type_keyed_str,
        Parrot_Complex_ro_unshift_float,
        Parrot_Complex_ro_unshift_integer,
        Parrot_Complex_ro_unshift_pmc,
        Parrot_Complex_ro_unshift_string,
        Parrot_default_visit
    };

    const  MMD_init _temp_mmd_init[] = {
        { MMD_ADD, 0, enum_type_PMC, (funcptr_t) Parrot_Complex_add },
        { MMD_ADD, 0, enum_class_Complex, (funcptr_t) Parrot_Complex_add_Complex },
        { MMD_ADD, 0, enum_type_INTVAL, (funcptr_t) Parrot_Complex_add_int },
        { MMD_ADD, 0, enum_type_FLOATVAL, (funcptr_t) Parrot_Complex_add_float },
        { MMD_I_ADD, 0, enum_type_PMC, (funcptr_t) Parrot_Complex_i_add },
        { MMD_I_ADD, 0, enum_class_Complex, (funcptr_t) Parrot_Complex_i_add_Complex },
        { MMD_I_ADD, 0, enum_type_INTVAL, (funcptr_t) Parrot_Complex_i_add_int },
        { MMD_I_ADD, 0, enum_type_FLOATVAL, (funcptr_t) Parrot_Complex_i_add_float },
        { MMD_SUBTRACT, 0, enum_type_PMC, (funcptr_t) Parrot_Complex_subtract },
        { MMD_SUBTRACT, 0, enum_class_Complex, (funcptr_t) Parrot_Complex_subtract_Complex },
        { MMD_SUBTRACT, 0, enum_type_INTVAL, (funcptr_t) Parrot_Complex_subtract_int },
        { MMD_SUBTRACT, 0, enum_type_FLOATVAL, (funcptr_t) Parrot_Complex_subtract_float },
        { MMD_I_SUBTRACT, 0, enum_type_PMC, (funcptr_t) Parrot_Complex_i_subtract },
        { MMD_I_SUBTRACT, 0, enum_class_Complex, (funcptr_t) Parrot_Complex_i_subtract_Complex },
        { MMD_I_SUBTRACT, 0, enum_type_INTVAL, (funcptr_t) Parrot_Complex_i_subtract_int },
        { MMD_I_SUBTRACT, 0, enum_type_FLOATVAL, (funcptr_t) Parrot_Complex_i_subtract_float },
        { MMD_MULTIPLY, 0, enum_type_PMC, (funcptr_t) Parrot_Complex_multiply },
        { MMD_MULTIPLY, 0, enum_class_Complex, (funcptr_t) Parrot_Complex_multiply_Complex },
        { MMD_MULTIPLY, 0, enum_type_INTVAL, (funcptr_t) Parrot_Complex_multiply_int },
        { MMD_MULTIPLY, 0, enum_type_FLOATVAL, (funcptr_t) Parrot_Complex_multiply_float },
        { MMD_I_MULTIPLY, 0, enum_type_PMC, (funcptr_t) Parrot_Complex_i_multiply },
        { MMD_I_MULTIPLY, 0, enum_class_Complex, (funcptr_t) Parrot_Complex_i_multiply_Complex },
        { MMD_I_MULTIPLY, 0, enum_type_INTVAL, (funcptr_t) Parrot_Complex_i_multiply_int },
        { MMD_I_MULTIPLY, 0, enum_type_FLOATVAL, (funcptr_t) Parrot_Complex_i_multiply_float },
        { MMD_DIVIDE, 0, enum_type_PMC, (funcptr_t) Parrot_Complex_divide },
        { MMD_DIVIDE, 0, enum_class_Complex, (funcptr_t) Parrot_Complex_divide_Complex },
        { MMD_DIVIDE, 0, enum_type_INTVAL, (funcptr_t) Parrot_Complex_divide_int },
        { MMD_DIVIDE, 0, enum_type_FLOATVAL, (funcptr_t) Parrot_Complex_divide_float },
        { MMD_I_DIVIDE, 0, enum_type_PMC, (funcptr_t) Parrot_Complex_i_divide },
        { MMD_I_DIVIDE, 0, enum_class_Complex, (funcptr_t) Parrot_Complex_i_divide_Complex },
        { MMD_I_DIVIDE, 0, enum_type_INTVAL, (funcptr_t) Parrot_Complex_i_divide_int },
        { MMD_I_DIVIDE, 0, enum_type_FLOATVAL, (funcptr_t) Parrot_Complex_i_divide_float },
        { MMD_POW, 0, enum_type_PMC, (funcptr_t) Parrot_Complex_pow },
        { MMD_POW, 0, enum_class_Complex, (funcptr_t) Parrot_Complex_pow_Complex },
        { MMD_EQ, 0, enum_type_PMC, (funcptr_t) Parrot_Complex_is_equal },
        { MMD_EQ, 0, enum_class_Complex, (funcptr_t) Parrot_Complex_is_equal_Complex }
    };
    /*  Dynamic PMCs need the runtime type
    which is passed in entry to class_init.
    */
    if (pass == 0) {
        /*
         * create vtable - clone it - we have to set a few items
         */
        VTABLE *vt_clone =
            Parrot_clone_vtable(interp, &temp_base_vtable);
        VTABLE *vt_ro_clone =
            Parrot_clone_vtable(interp, &temp_ro_vtable);
        vt_clone->whoami = CONST_STRING(interp, "Complex");
        vt_clone->isa_str = CONST_STRING(interp, "Complex");
        vt_clone->does_str = CONST_STRING(interp, "scalar");
        vt_ro_clone->base_type = entry;
        vt_ro_clone->whoami = vt_clone->whoami;
        vt_ro_clone->isa_str = vt_clone->isa_str;
        vt_ro_clone->does_str = vt_clone->does_str;
        vt_clone->ro_variant_vtable = vt_ro_clone;
        vt_ro_clone->ro_variant_vtable = vt_clone;
        interp->vtables[entry] = vt_clone;
    }
    else { /* pass */
        /* setup MRO and _namespace */
        Parrot_create_mro(interp, entry);
        /* create PMC Proxy object */
        Parrot_create_pmc_proxy(interp, entry);
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_ln),
                "ln", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_exp),
                "exp", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_sin),
                "sin", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_cos),
                "cos", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_tan),
                "tan", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_cot),
                "cot", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_sec),
                "sec", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_csc),
                "csc", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_asin),
                "asin", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_acos),
                "acos", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_atan),
                "atan", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_acot),
                "acot", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_acsc),
                "acsc", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_asec),
                "asec", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_sinh),
                "sinh", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_cosh),
                "cosh", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_tanh),
                "tanh", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_coth),
                "coth", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_csch),
                "csch", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_sech),
                "sech", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_asinh),
                "asinh", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_acosh),
                "acosh", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_atanh),
                "atanh", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_acoth),
                "acoth", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_acsch),
                "acsch", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_asech),
                "asech", "PJO");
        register_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_nci_sqrt),
                "sqrt", "PJO");
        /* class_init */
        {
#define N_MMD_INIT (sizeof(_temp_mmd_init)/sizeof(_temp_mmd_init[0]))
            Parrot_mmd_register_table(interp, entry,
                _temp_mmd_init, N_MMD_INIT);
        }
    } /* pass */
} /* Parrot_Complex_class_init */


/*

=back

=cut

*/

/*
 * Local variables:
 *   c-file-style: "parrot"
 * End:
 * vim: expandtab shiftwidth=4:
 */
