/*
 * !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 *
 * This file is generated automatically from './src/pmc/timer.pmc'
 * by tools/build/pmc2c.pl.
 *
 * Any changes made here will be lost!
 *
 */

#line 1 "./src/pmc/timer.pmc"
/*
Copyright (C) 2001-2007, The Perl Foundation.
$Id: timer.pmc 19029 2007-06-16 03:43:55Z petdance $

=head1 NAME

src/pmc/timer.pmc - Timer

=head1 SYNOPSIS

    new P0, .Timer
    set P0[.PARROT_TIMER_SEC], I_seconds        # whole seconds
    set P0[.PARROT_TIMER_USEC], I_micro seconds # and/or micro seconds
    set P0[.PARROT_TIMER_NSEC], N_seconds_frac  # or fraction in seconds
    set P0[.PARROT_TIMER_REPEAT], I_repeat      # 0 = one shot ...
    set P0[.PARROT_TIMER_HANDLER], P_sub        # set handler sub PMC
    invoke P0                                   # start timer
    set P0[.PARROT_TIMER_RUNNING], 1            # same

    new P0, .Timer, P1                          # set everything

    set I0, P0[.PARROT_TIMER_SEC]               # query current timer status
    set N0, P0[.PARROT_TIMER_NSEC]
    ...
    set P0[.PARROT_TIMER_RUNNING], 0            # turn timer off


=head1 DESCRIPTION

This is the Timer base class

Running timers are kept in a linked list. Each timer has a tick count,
which gets decremented if the system timer expires. If the tick count
reaches zero, the timer handler gets invoked via Parrot's event handling
code.

The Timer resolution is operating system dependent. It's only guaranteed
that the Timer will fire some time after the programmed interval.

The Timer stops after invoking the handler (repeat+1) times. To create a
Timer that will run forever, set "repeat" to -1. Turning the Timer off
preserves set values; the Timer is not destroyed.

When setting both C<PARROT_TIMER_SEC> and C<PARROT_TIMER_USEC> it must
be done in that sequence, whole seconds first. If a timer is constructed
with an initializer and it should be run immediately, the
C<PARROT_TIMER_RUNNING> value has to be in the last key of the
initializer.

=head2 Functions

=over 4

=cut

*/

#include "parrot/parrot.h"

/* interface constants */
/* &gen_from_enum(timer.pasm) */
typedef enum {
    PARROT_TIMER_SEC,
    PARROT_TIMER_USEC,
    PARROT_TIMER_NSEC,
    PARROT_TIMER_REPEAT,
    PARROT_TIMER_RUNNING,
    PARROT_TIMER_HANDLER,
    PARROT_TIMER_MAX
} parrot_timer_enum_t;
/* &end_gen */

/*

=item C<static void
add_timer(Interp* interp, PMC *pmc)>

Adds a new timer event to the event queue.

=cut

*/

static void
add_timer(Interp* interp, PMC *pmc)
{
    const parrot_timer_event * const t = (parrot_timer_event *)PMC_struct_val(pmc);

    if (!PMC_IS_NULL(t->sub))
        Parrot_new_timer_event(interp, pmc, t->abs_time,
            t->interval, t->repeat, t->sub, EVENT_TYPE_TIMER);
}

/*

=item C<static void
del_timer(Interp* interp, PMC *pmc)>

Deletes the timer event.

=cut

*/

static void
del_timer(Interp* interp, PMC *pmc)
{
   Parrot_del_timer_event(interp, pmc);
}


#line 124 "src/pmc/timer.c"
#include "parrot/parrot.h"
#include "parrot/extend.h"
#include "parrot/dynext.h"
#include "pmc_timer.h"
#include "pmc_default.h"
#include "timer.str"

void
Parrot_Timer_destroy(Interp *interp, PMC* pmc)
#line 188 "./src/pmc/timer.pmc"
{
    parrot_timer_event * const t = (parrot_timer_event *)PMC_struct_val(pmc);
    del_timer(interp, pmc);
    mem_sys_free(t);

}
#line 141 "src/pmc/timer.c"
INTVAL
Parrot_Timer_get_integer_keyed_int(Interp *interp, PMC* pmc, INTVAL key)
#line 204 "./src/pmc/timer.pmc"
{
    parrot_timer_event * const t = (parrot_timer_event *)PMC_struct_val(pmc);

    switch (key) {
        case PARROT_TIMER_SEC:
            return (INTVAL)t->abs_time;
        case PARROT_TIMER_USEC:
            return (INTVAL)((t->abs_time - (INTVAL)t->abs_time) *1000000.0);
        case PARROT_TIMER_REPEAT:
            return (INTVAL) t->repeat;
        case PARROT_TIMER_RUNNING:
            return 0;
    }
    return -1;

}
#line 161 "src/pmc/timer.c"
FLOATVAL
Parrot_Timer_get_number_keyed_int(Interp *interp, PMC* pmc, INTVAL key)
#line 251 "./src/pmc/timer.pmc"
{
    parrot_timer_event * const t = (parrot_timer_event *)PMC_struct_val(pmc);

    switch (key) {
        case PARROT_TIMER_NSEC:
            return t->abs_time;
    }

    return -1.0;

}
#line 176 "src/pmc/timer.c"
PMC*
Parrot_Timer_get_pmc_keyed_int(Interp *interp, PMC* pmc, INTVAL key)
#line 230 "./src/pmc/timer.pmc"
{
    parrot_timer_event * const t = (parrot_timer_event *)PMC_struct_val(pmc);

    switch (key) {
        case PARROT_TIMER_HANDLER:
            return t->sub;
    }

    return PMCNULL;

}
#line 191 "src/pmc/timer.c"
void
Parrot_Timer_init(Interp *interp, PMC* pmc)
#line 124 "./src/pmc/timer.pmc"
{
    /*
     * t->abs_time actually is relative, its converted to an
     * absolute time, when the timer is started
     */
    parrot_timer_event * const t = mem_allocate_zeroed_typed(parrot_timer_event);
    PMC_struct_val(pmc)  = t;
    PObj_active_destroy_SET(pmc);
    ++interp->arena_base->num_early_DOD_PMCs;

}
#line 206 "src/pmc/timer.c"
void
Parrot_Timer_init_pmc(Interp *interp, PMC* pmc, PMC *init)
#line 145 "./src/pmc/timer.pmc"
{
    const INTVAL n = VTABLE_get_integer(interp, init);
    INTVAL i;

    Parrot_Timer_init(interp, pmc);
    for (i = 0; i < n; i += 2) {
        const INTVAL key = VTABLE_get_integer_keyed_int(interp, init, i);
        switch (key) {
            case PARROT_TIMER_SEC:
            case PARROT_TIMER_USEC:
            case PARROT_TIMER_REPEAT:
            case PARROT_TIMER_RUNNING:
                {
                const INTVAL val = VTABLE_get_integer_keyed_int(interp, init, i+1);
                Parrot_Timer_set_integer_keyed_int(interp, pmc, key, val);
                }
                break;
            case PARROT_TIMER_NSEC:
                {
                const FLOATVAL nval = VTABLE_get_number_keyed_int(interp, init, i+1);
                Parrot_Timer_set_number_keyed_int(interp, pmc, key, nval);
                }
                break;
            case PARROT_TIMER_HANDLER:
                {
                PMC * const pval = VTABLE_get_pmc_keyed_int(interp, init, i+1);
                Parrot_Timer_set_pmc_keyed_int(interp, pmc, key, pval);
                }
                break;
        }
    }

}
#line 243 "src/pmc/timer.c"
opcode_t*
Parrot_Timer_invoke(Interp *interp, PMC* pmc, void *next)
#line 326 "./src/pmc/timer.pmc"
{
    add_timer(interp, pmc);
    return (opcode_t *)next;

}
#line 252 "src/pmc/timer.c"
void
Parrot_Timer_set_integer_keyed_int(Interp *interp, PMC* pmc, INTVAL key, INTVAL value)
#line 272 "./src/pmc/timer.pmc"
{
    parrot_timer_event * const t = (parrot_timer_event *)PMC_struct_val(pmc);

    switch (key) {
        case PARROT_TIMER_SEC:
            t->abs_time = value;
            break;
        case PARROT_TIMER_USEC:
            t->abs_time += value / 1000000.0;
            break;
        case PARROT_TIMER_REPEAT:
            t->repeat = value;
            break;
        case PARROT_TIMER_RUNNING:
            if (value) {
                add_timer(interp, pmc);
            }
            else {
                del_timer(interp, pmc);
            }
            break;
    }

}
#line 280 "src/pmc/timer.c"
void
Parrot_Timer_set_number_keyed_int(Interp *interp, PMC* pmc, INTVAL key, FLOATVAL value)
#line 341 "./src/pmc/timer.pmc"
{
    parrot_timer_event * const t = (parrot_timer_event *)PMC_struct_val(pmc);

    switch (key) {
        case PARROT_TIMER_NSEC:
            t->abs_time = value;
            break;
    }

}
#line 294 "src/pmc/timer.c"
void
Parrot_Timer_set_pmc_keyed_int(Interp *interp, PMC* pmc, INTVAL key, PMC* value)
#line 306 "./src/pmc/timer.pmc"
{
    parrot_timer_event * const t = (parrot_timer_event *)PMC_struct_val(pmc);

    switch (key) {
        case PARROT_TIMER_HANDLER:
            t->sub = value;
            break;
    }

}
#line 308 "src/pmc/timer.c"
void
Parrot_Timer_ro_add_attribute(Interp *interp, PMC* pmc, STRING* name, PMC* type)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "add_attribute() in read-only instance of Timer");
}
void
Parrot_Timer_ro_add_method(Interp *interp, PMC* pmc, STRING* method_name, PMC* sub_pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "add_method() in read-only instance of Timer");
}
void
Parrot_Timer_ro_add_parent(Interp *interp, PMC* pmc, PMC* parent)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "add_parent() in read-only instance of Timer");
}
void
Parrot_Timer_ro_add_role(Interp *interp, PMC* pmc, PMC* role)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "add_role() in read-only instance of Timer");
}
void
Parrot_Timer_ro_assign_pmc(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "assign_pmc() in read-only instance of Timer");
}
void
Parrot_Timer_ro_assign_string_native(Interp *interp, PMC* pmc, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "assign_string_native() in read-only instance of Timer");
}
void
Parrot_Timer_ro_decrement(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "decrement() in read-only instance of Timer");
}
void
Parrot_Timer_ro_delete_keyed(Interp *interp, PMC* pmc, PMC* key)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "delete_keyed() in read-only instance of Timer");
}
void
Parrot_Timer_ro_delete_keyed_int(Interp *interp, PMC* pmc, INTVAL key)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "delete_keyed_int() in read-only instance of Timer");
}
void
Parrot_Timer_ro_delete_keyed_str(Interp *interp, PMC* pmc, STRING* key)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "delete_keyed_str() in read-only instance of Timer");
}
PMC*
Parrot_Timer_ro_find_method(Interp *interp, PMC* pmc, STRING* method_name)
 {
    PMC *const method = Parrot_default_find_method(interp, pmc, method_name);
    if (!PMC_IS_NULL(VTABLE_getprop(interp, method, const_string(interp, "write"))))
        return PMCNULL;
    else
        return method;
}
void
Parrot_Timer_ro_i_absolute(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "i_absolute() in read-only instance of Timer");
}
void
Parrot_Timer_ro_i_bitwise_not(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "i_bitwise_not() in read-only instance of Timer");
}
void
Parrot_Timer_ro_i_bitwise_nots(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "i_bitwise_nots() in read-only instance of Timer");
}
void
Parrot_Timer_ro_i_logical_not(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "i_logical_not() in read-only instance of Timer");
}
void
Parrot_Timer_ro_i_neg(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "i_neg() in read-only instance of Timer");
}
void
Parrot_Timer_ro_increment(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "increment() in read-only instance of Timer");
}
void
Parrot_Timer_ro_morph(Interp *interp, PMC* pmc, INTVAL type)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "morph() in read-only instance of Timer");
}
FLOATVAL
Parrot_Timer_ro_pop_float(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "pop_float() in read-only instance of Timer");
    return (FLOATVAL)0;
}
INTVAL
Parrot_Timer_ro_pop_integer(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "pop_integer() in read-only instance of Timer");
    return (INTVAL)0;
}
PMC*
Parrot_Timer_ro_pop_pmc(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "pop_pmc() in read-only instance of Timer");
    return PMCNULL;
}
STRING*
Parrot_Timer_ro_pop_string(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "pop_string() in read-only instance of Timer");
    return (STRING*)0;
}
void
Parrot_Timer_ro_push_float(Interp *interp, PMC* pmc, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "push_float() in read-only instance of Timer");
}
void
Parrot_Timer_ro_push_integer(Interp *interp, PMC* pmc, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "push_integer() in read-only instance of Timer");
}
void
Parrot_Timer_ro_push_pmc(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "push_pmc() in read-only instance of Timer");
}
void
Parrot_Timer_ro_push_string(Interp *interp, PMC* pmc, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "push_string() in read-only instance of Timer");
}
void
Parrot_Timer_ro_remove_attribute(Interp *interp, PMC* pmc, STRING* name)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "remove_attribute() in read-only instance of Timer");
}
void
Parrot_Timer_ro_remove_method(Interp *interp, PMC* pmc, STRING* method_name)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "remove_method() in read-only instance of Timer");
}
void
Parrot_Timer_ro_remove_parent(Interp *interp, PMC* pmc, PMC* parent)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "remove_parent() in read-only instance of Timer");
}
void
Parrot_Timer_ro_remove_role(Interp *interp, PMC* pmc, PMC* role)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "remove_role() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_attr(Interp *interp, PMC* pmc, INTVAL idx, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_attr() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_attr_str(Interp *interp, PMC* pmc, STRING* idx, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_attr_str() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_bignum_int(Interp *interp, PMC* pmc, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_bignum_int() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_bignum_num(Interp *interp, PMC* pmc, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_bignum_num() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_bignum_str(Interp *interp, PMC* pmc, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_bignum_str() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_bool(Interp *interp, PMC* pmc, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_bool() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_integer_keyed(Interp *interp, PMC* pmc, PMC* key, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_integer_keyed() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_integer_keyed_int(Interp *interp, PMC* pmc, INTVAL key, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_integer_keyed_int() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_integer_keyed_str(Interp *interp, PMC* pmc, STRING* key, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_integer_keyed_str() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_integer_native(Interp *interp, PMC* pmc, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_integer_native() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_integer_same(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_integer_same() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_number_keyed(Interp *interp, PMC* pmc, PMC* key, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_number_keyed() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_number_keyed_int(Interp *interp, PMC* pmc, INTVAL key, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_number_keyed_int() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_number_keyed_str(Interp *interp, PMC* pmc, STRING* key, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_number_keyed_str() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_number_native(Interp *interp, PMC* pmc, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_number_native() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_number_same(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_number_same() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_pmc(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pmc() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_pmc_keyed(Interp *interp, PMC* pmc, PMC* key, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pmc_keyed() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_pmc_keyed_int(Interp *interp, PMC* pmc, INTVAL key, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pmc_keyed_int() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_pmc_keyed_str(Interp *interp, PMC* pmc, STRING* key, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pmc_keyed_str() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_pointer(Interp *interp, PMC* pmc, void* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pointer() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_pointer_keyed(Interp *interp, PMC* pmc, PMC* key, void* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pointer_keyed() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_pointer_keyed_int(Interp *interp, PMC* pmc, INTVAL key, void* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pointer_keyed_int() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_pointer_keyed_str(Interp *interp, PMC* pmc, STRING* key, void* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_pointer_keyed_str() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_string_keyed(Interp *interp, PMC* pmc, PMC* key, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_string_keyed() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_string_keyed_int(Interp *interp, PMC* pmc, INTVAL key, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_string_keyed_int() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_string_keyed_str(Interp *interp, PMC* pmc, STRING* key, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_string_keyed_str() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_string_native(Interp *interp, PMC* pmc, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_string_native() in read-only instance of Timer");
}
void
Parrot_Timer_ro_set_string_same(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "set_string_same() in read-only instance of Timer");
}
FLOATVAL
Parrot_Timer_ro_shift_float(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "shift_float() in read-only instance of Timer");
    return (FLOATVAL)0;
}
INTVAL
Parrot_Timer_ro_shift_integer(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "shift_integer() in read-only instance of Timer");
    return (INTVAL)0;
}
PMC*
Parrot_Timer_ro_shift_pmc(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "shift_pmc() in read-only instance of Timer");
    return PMCNULL;
}
STRING*
Parrot_Timer_ro_shift_string(Interp *interp, PMC* pmc)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "shift_string() in read-only instance of Timer");
    return (STRING*)0;
}
void
Parrot_Timer_ro_splice(Interp *interp, PMC* pmc, PMC* value, INTVAL offset, INTVAL count)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "splice() in read-only instance of Timer");
}
void
Parrot_Timer_ro_thaw(Interp *interp, PMC* pmc, visit_info* info)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "thaw() in read-only instance of Timer");
}
void
Parrot_Timer_ro_thawfinish(Interp *interp, PMC* pmc, visit_info* info)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "thawfinish() in read-only instance of Timer");
}
void
Parrot_Timer_ro_unshift_float(Interp *interp, PMC* pmc, FLOATVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "unshift_float() in read-only instance of Timer");
}
void
Parrot_Timer_ro_unshift_integer(Interp *interp, PMC* pmc, INTVAL value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "unshift_integer() in read-only instance of Timer");
}
void
Parrot_Timer_ro_unshift_pmc(Interp *interp, PMC* pmc, PMC* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "unshift_pmc() in read-only instance of Timer");
}
void
Parrot_Timer_ro_unshift_string(Interp *interp, PMC* pmc, STRING* value)
 {
    internal_exception(WRITE_TO_CONSTCLASS,
            "unshift_string() in read-only instance of Timer");
}
void
Parrot_Timer_class_init(Parrot_Interp interp, int entry, int pass)
{
    const VTABLE temp_base_vtable = {
        NULL, /* namespace */
        enum_class_Timer, /* base_type */
        NULL, /* whoami */
        0|VTABLE_HAS_READONLY_FLAG, /* flags */
        NULL,   /* does_str */
        NULL,   /* isa_str */
        NULL,   /* class */
        NULL,   /* mro */
        NULL,   /* ro_variant_vtable */
        Parrot_default_absolute,
        Parrot_default_add_attribute,
        Parrot_default_add_method,
        Parrot_default_add_parent,
        Parrot_default_add_role,
        Parrot_default_assign_pmc,
        Parrot_default_assign_string_native,
        Parrot_default_bitwise_not,
        Parrot_default_bitwise_nots,
        Parrot_default_can,
        Parrot_default_class_type,
        Parrot_default_clone,
        Parrot_default_clone_pmc,
        Parrot_default_decrement,
        Parrot_default_defined,
        Parrot_default_defined_keyed,
        Parrot_default_defined_keyed_int,
        Parrot_default_defined_keyed_str,
        Parrot_default_delete_keyed,
        Parrot_default_delete_keyed_int,
        Parrot_default_delete_keyed_str,
        Parrot_default_delprop,
        Parrot_Timer_destroy,
        Parrot_default_does,
        Parrot_default_elements,
        Parrot_default_exists_keyed,
        Parrot_default_exists_keyed_int,
        Parrot_default_exists_keyed_str,
        Parrot_default_find_method,
        Parrot_default_freeze,
        Parrot_default_get_attr,
        Parrot_default_get_attr_str,
        Parrot_default_get_bignum,
        Parrot_default_get_bool,
        Parrot_default_get_class,
        Parrot_default_get_integer,
        Parrot_default_get_integer_keyed,
        Parrot_Timer_get_integer_keyed_int,
        Parrot_default_get_integer_keyed_str,
        Parrot_default_get_iter,
        Parrot_default_get_number,
        Parrot_default_get_number_keyed,
        Parrot_Timer_get_number_keyed_int,
        Parrot_default_get_number_keyed_str,
        Parrot_default_get_pmc,
        Parrot_default_get_pmc_keyed,
        Parrot_Timer_get_pmc_keyed_int,
        Parrot_default_get_pmc_keyed_str,
        Parrot_default_get_pointer,
        Parrot_default_get_pointer_keyed,
        Parrot_default_get_pointer_keyed_int,
        Parrot_default_get_pointer_keyed_str,
        Parrot_default_get_repr,
        Parrot_default_get_string,
        Parrot_default_get_string_keyed,
        Parrot_default_get_string_keyed_int,
        Parrot_default_get_string_keyed_str,
        Parrot_default_getprop,
        Parrot_default_getprops,
        Parrot_default_hash,
        Parrot_default_i_absolute,
        Parrot_default_i_bitwise_not,
        Parrot_default_i_bitwise_nots,
        Parrot_default_i_logical_not,
        Parrot_default_i_neg,
        Parrot_default_increment,
        Parrot_Timer_init,
        Parrot_Timer_init_pmc,
        Parrot_default_inspect,
        Parrot_default_inspect_str,
        Parrot_default_instantiate,
        Parrot_Timer_invoke,
        Parrot_default_is_same,
        Parrot_default_isa,
        Parrot_default_logical_not,
        Parrot_default_mark,
        Parrot_default_morph,
        Parrot_default_name,
        Parrot_default_neg,
        Parrot_default_new_from_string,
        Parrot_default_nextkey_keyed,
        Parrot_default_nextkey_keyed_int,
        Parrot_default_nextkey_keyed_str,
        Parrot_default_pmc_namespace,
        Parrot_default_pop_float,
        Parrot_default_pop_integer,
        Parrot_default_pop_pmc,
        Parrot_default_pop_string,
        Parrot_default_push_float,
        Parrot_default_push_integer,
        Parrot_default_push_pmc,
        Parrot_default_push_string,
        Parrot_default_remove_attribute,
        Parrot_default_remove_method,
        Parrot_default_remove_parent,
        Parrot_default_remove_role,
        Parrot_default_set_attr,
        Parrot_default_set_attr_str,
        Parrot_default_set_bignum_int,
        Parrot_default_set_bignum_num,
        Parrot_default_set_bignum_str,
        Parrot_default_set_bool,
        Parrot_default_set_integer_keyed,
        Parrot_Timer_set_integer_keyed_int,
        Parrot_default_set_integer_keyed_str,
        Parrot_default_set_integer_native,
        Parrot_default_set_integer_same,
        Parrot_default_set_number_keyed,
        Parrot_Timer_set_number_keyed_int,
        Parrot_default_set_number_keyed_str,
        Parrot_default_set_number_native,
        Parrot_default_set_number_same,
        Parrot_default_set_pmc,
        Parrot_default_set_pmc_keyed,
        Parrot_Timer_set_pmc_keyed_int,
        Parrot_default_set_pmc_keyed_str,
        Parrot_default_set_pointer,
        Parrot_default_set_pointer_keyed,
        Parrot_default_set_pointer_keyed_int,
        Parrot_default_set_pointer_keyed_str,
        Parrot_default_set_string_keyed,
        Parrot_default_set_string_keyed_int,
        Parrot_default_set_string_keyed_str,
        Parrot_default_set_string_native,
        Parrot_default_set_string_same,
        Parrot_default_setprop,
        Parrot_default_share,
        Parrot_default_share_ro,
        Parrot_default_shift_float,
        Parrot_default_shift_integer,
        Parrot_default_shift_pmc,
        Parrot_default_shift_string,
        Parrot_default_slice,
        Parrot_default_splice,
        Parrot_default_subclass,
        Parrot_default_substr,
        Parrot_default_substr_str,
        Parrot_default_subtype,
        Parrot_default_thaw,
        Parrot_default_thawfinish,
        Parrot_default_type,
        Parrot_default_type_keyed,
        Parrot_default_type_keyed_int,
        Parrot_default_type_keyed_str,
        Parrot_default_unshift_float,
        Parrot_default_unshift_integer,
        Parrot_default_unshift_pmc,
        Parrot_default_unshift_string,
        Parrot_default_visit
    };

    const VTABLE temp_ro_vtable = {
        NULL, /* namespace */
        enum_class_Timer, /* base_type */
        NULL, /* whoami */
        0|VTABLE_IS_READONLY_FLAG, /* flags */
        NULL,   /* does_str */
        NULL,   /* isa_str */
        NULL,   /* class */
        NULL,   /* mro */
        NULL,   /* ro_variant_vtable */
        Parrot_default_absolute,
        Parrot_Timer_ro_add_attribute,
        Parrot_Timer_ro_add_method,
        Parrot_Timer_ro_add_parent,
        Parrot_Timer_ro_add_role,
        Parrot_Timer_ro_assign_pmc,
        Parrot_Timer_ro_assign_string_native,
        Parrot_default_bitwise_not,
        Parrot_default_bitwise_nots,
        Parrot_default_can,
        Parrot_default_class_type,
        Parrot_default_clone,
        Parrot_default_clone_pmc,
        Parrot_Timer_ro_decrement,
        Parrot_default_defined,
        Parrot_default_defined_keyed,
        Parrot_default_defined_keyed_int,
        Parrot_default_defined_keyed_str,
        Parrot_Timer_ro_delete_keyed,
        Parrot_Timer_ro_delete_keyed_int,
        Parrot_Timer_ro_delete_keyed_str,
        Parrot_default_delprop,
        Parrot_Timer_destroy,
        Parrot_default_does,
        Parrot_default_elements,
        Parrot_default_exists_keyed,
        Parrot_default_exists_keyed_int,
        Parrot_default_exists_keyed_str,
        Parrot_Timer_ro_find_method,
        Parrot_default_freeze,
        Parrot_default_get_attr,
        Parrot_default_get_attr_str,
        Parrot_default_get_bignum,
        Parrot_default_get_bool,
        Parrot_default_get_class,
        Parrot_default_get_integer,
        Parrot_default_get_integer_keyed,
        Parrot_Timer_get_integer_keyed_int,
        Parrot_default_get_integer_keyed_str,
        Parrot_default_get_iter,
        Parrot_default_get_number,
        Parrot_default_get_number_keyed,
        Parrot_Timer_get_number_keyed_int,
        Parrot_default_get_number_keyed_str,
        Parrot_default_get_pmc,
        Parrot_default_get_pmc_keyed,
        Parrot_Timer_get_pmc_keyed_int,
        Parrot_default_get_pmc_keyed_str,
        Parrot_default_get_pointer,
        Parrot_default_get_pointer_keyed,
        Parrot_default_get_pointer_keyed_int,
        Parrot_default_get_pointer_keyed_str,
        Parrot_default_get_repr,
        Parrot_default_get_string,
        Parrot_default_get_string_keyed,
        Parrot_default_get_string_keyed_int,
        Parrot_default_get_string_keyed_str,
        Parrot_default_getprop,
        Parrot_default_getprops,
        Parrot_default_hash,
        Parrot_Timer_ro_i_absolute,
        Parrot_Timer_ro_i_bitwise_not,
        Parrot_Timer_ro_i_bitwise_nots,
        Parrot_Timer_ro_i_logical_not,
        Parrot_Timer_ro_i_neg,
        Parrot_Timer_ro_increment,
        Parrot_Timer_init,
        Parrot_Timer_init_pmc,
        Parrot_default_inspect,
        Parrot_default_inspect_str,
        Parrot_default_instantiate,
        Parrot_Timer_invoke,
        Parrot_default_is_same,
        Parrot_default_isa,
        Parrot_default_logical_not,
        Parrot_default_mark,
        Parrot_Timer_ro_morph,
        Parrot_default_name,
        Parrot_default_neg,
        Parrot_default_new_from_string,
        Parrot_default_nextkey_keyed,
        Parrot_default_nextkey_keyed_int,
        Parrot_default_nextkey_keyed_str,
        Parrot_default_pmc_namespace,
        Parrot_Timer_ro_pop_float,
        Parrot_Timer_ro_pop_integer,
        Parrot_Timer_ro_pop_pmc,
        Parrot_Timer_ro_pop_string,
        Parrot_Timer_ro_push_float,
        Parrot_Timer_ro_push_integer,
        Parrot_Timer_ro_push_pmc,
        Parrot_Timer_ro_push_string,
        Parrot_Timer_ro_remove_attribute,
        Parrot_Timer_ro_remove_method,
        Parrot_Timer_ro_remove_parent,
        Parrot_Timer_ro_remove_role,
        Parrot_Timer_ro_set_attr,
        Parrot_Timer_ro_set_attr_str,
        Parrot_Timer_ro_set_bignum_int,
        Parrot_Timer_ro_set_bignum_num,
        Parrot_Timer_ro_set_bignum_str,
        Parrot_Timer_ro_set_bool,
        Parrot_Timer_ro_set_integer_keyed,
        Parrot_Timer_ro_set_integer_keyed_int,
        Parrot_Timer_ro_set_integer_keyed_str,
        Parrot_Timer_ro_set_integer_native,
        Parrot_Timer_ro_set_integer_same,
        Parrot_Timer_ro_set_number_keyed,
        Parrot_Timer_ro_set_number_keyed_int,
        Parrot_Timer_ro_set_number_keyed_str,
        Parrot_Timer_ro_set_number_native,
        Parrot_Timer_ro_set_number_same,
        Parrot_Timer_ro_set_pmc,
        Parrot_Timer_ro_set_pmc_keyed,
        Parrot_Timer_ro_set_pmc_keyed_int,
        Parrot_Timer_ro_set_pmc_keyed_str,
        Parrot_Timer_ro_set_pointer,
        Parrot_Timer_ro_set_pointer_keyed,
        Parrot_Timer_ro_set_pointer_keyed_int,
        Parrot_Timer_ro_set_pointer_keyed_str,
        Parrot_Timer_ro_set_string_keyed,
        Parrot_Timer_ro_set_string_keyed_int,
        Parrot_Timer_ro_set_string_keyed_str,
        Parrot_Timer_ro_set_string_native,
        Parrot_Timer_ro_set_string_same,
        Parrot_default_setprop,
        Parrot_default_share,
        Parrot_default_share_ro,
        Parrot_Timer_ro_shift_float,
        Parrot_Timer_ro_shift_integer,
        Parrot_Timer_ro_shift_pmc,
        Parrot_Timer_ro_shift_string,
        Parrot_default_slice,
        Parrot_Timer_ro_splice,
        Parrot_default_subclass,
        Parrot_default_substr,
        Parrot_default_substr_str,
        Parrot_default_subtype,
        Parrot_Timer_ro_thaw,
        Parrot_Timer_ro_thawfinish,
        Parrot_default_type,
        Parrot_default_type_keyed,
        Parrot_default_type_keyed_int,
        Parrot_default_type_keyed_str,
        Parrot_Timer_ro_unshift_float,
        Parrot_Timer_ro_unshift_integer,
        Parrot_Timer_ro_unshift_pmc,
        Parrot_Timer_ro_unshift_string,
        Parrot_default_visit
    };
    if (pass == 0) {
        /*
         * create vtable - clone it - we have to set a few items
         */
        VTABLE *vt_clone =
            Parrot_clone_vtable(interp, &temp_base_vtable);
        VTABLE *vt_ro_clone =
            Parrot_clone_vtable(interp, &temp_ro_vtable);
        vt_clone->whoami = CONST_STRING(interp, "Timer");
        vt_clone->isa_str = CONST_STRING(interp, "Timer");
        vt_clone->does_str = CONST_STRING(interp, "event");
        vt_ro_clone->base_type = entry;
        vt_ro_clone->whoami = vt_clone->whoami;
        vt_ro_clone->isa_str = vt_clone->isa_str;
        vt_ro_clone->does_str = vt_clone->does_str;
        vt_clone->ro_variant_vtable = vt_ro_clone;
        vt_ro_clone->ro_variant_vtable = vt_clone;
        interp->vtables[entry] = vt_clone;
    }
    else { /* pass */
        /* setup MRO and _namespace */
        Parrot_create_mro(interp, entry);
        /* create PMC Proxy object */
        Parrot_create_pmc_proxy(interp, entry);
        /* class_init */
        {
        }
    } /* pass */
} /* Parrot_Timer_class_init */


/*

=back

=head1 HISTORY

Initial proposal by leo 2003.07.15

Initial revision 2003.07.17

Switch to events 2004.01.07

=cut

*/

/*
 * Local variables:
 *   c-file-style: "parrot"
 * End:
 * vim: expandtab shiftwidth=4:
 */
